@@ -16,20 +16,33 @@ using System.Security.Permissions;
 namespace Mono.Data.SybaseClient {
 	public sealed class SybasePermission : DBDataPermission 
 	{
+#if NET_2_0
+		[Obsolete ("Use SybasePermission(PermissionState.None)", true)]
+#endif
 		[MonoTODO]
 		public SybasePermission () 
+#if NET_2_0
+			: base (PermissionState.None)
+#endif
 		{
 			// FIXME: do constructor
 		}
 
 		[MonoTODO]
 		public SybasePermission (PermissionState state) 
+			: base (state)
 		{
 			// FIXME: do constructor
 		}
 
+#if NET_2_0
+		[Obsolete ("Use SybasePermission(PermissionState.None)", true)]
+#endif
 		[MonoTODO]
 		public SybasePermission (PermissionState state, bool allowBlankPassword) 
+#if NET_2_0
+			: base (PermissionState.None)
+#endif
 		{ 
 			throw new NotImplementedException ();
 		}
