@@ -3,10 +3,15 @@
 //
 // Author:
 //   Tim Coleman (tim@timcoleman.com)
+//   Sebastien Pouliot (spouliot@motus.com)
+//   Daniel Morgan (danielmorgan@verizon.net)
 //
 // Copyright (C) 2002 Tim Coleman
+// Portions (C) 2003 Motus Technologies Inc. (http://www.motus.com)
+// Portions (C) 2003 Daniel Morgan
 //
 
+using Mono.Security.Protocol.Ntlm;
 using System;
 using System.Collections;
 using System.ComponentModel;
@@ -21,6 +26,9 @@ namespace Mono.Data.Tds.Protocol {
 
 		TdsComm comm;
 		TdsVersion tdsVersion;
+		
+		protected internal TdsConnectionParameters connectionParms;
+		protected readonly byte[] NTLMSSP_ID = new byte[] {0x4e, 0x54, 0x4c, 0x4d, 0x53, 0x53, 0x50, 0x00};
 
 		int packetSize;
 		string dataSource;
@@ -36,7 +44,7 @@ namespace Mono.Data.Tds.Protocol {
 		bool moreResults;
 
 		Encoding encoder;
-		bool autoCommit;
+//		bool autoCommit;
 
 		bool doneProc;
 		TdsDataRow currentRow = null;
@@ -51,8 +59,8 @@ namespace Mono.Data.Tds.Protocol {
 		int cancelsRequested;
 		int cancelsProcessed;
 
-		bool isDone;
-		bool isDoneInProc;
+//		bool isDone;
+//		bool isDoneInProc;
 
 		ArrayList outputParameters = new ArrayList ();
 		protected TdsInternalErrorCollection messages = new TdsInternalErrorCollection ();
@@ -187,6 +195,11 @@ namespace Mono.Data.Tds.Protocol {
 			comm.Close ();
 			connected = false;
 		}
+		
+		public virtual bool Reset ()
+		{
+			return true;
+		}
 
 		public void Execute (string sql)
 		{
@@ -443,7 +456,7 @@ namespace Mono.Data.Tds.Protocol {
 				if (outParam) 
 					comm.Skip (1);
 				if (comm.GetByte () == 0)
-					element = null;
+					element = DBNull.Value;
 				else
 					element = (comm.GetByte() != 0);
 				break;
@@ -462,7 +475,7 @@ namespace Mono.Data.Tds.Protocol {
 				}
 				break;
 			default :
-				return null;
+				return DBNull.Value;
 			}
 
 			return element;
@@ -471,7 +484,7 @@ namespace Mono.Data.Tds.Protocol {
 		private object GetBinaryValue ()
 		{
 			int len;
-			object result = null;
+			object result = DBNull.Value;
 			if (tdsVersion == TdsVersion.tds70) {
 				len = comm.GetTdsShort ();
 				if (len != 0xffff && len > 0)
@@ -488,7 +501,7 @@ namespace Mono.Data.Tds.Protocol {
 		private object GetDateTimeValue (TdsColumnType type)
 		{
 			int len = 0;
-			object result = null;
+			object result;
 		
 			switch (type) {
 			case TdsColumnType.DateTime4:
@@ -523,6 +536,9 @@ namespace Mono.Data.Tds.Protocol {
 				if (minutes != 0) 
 					result = ((DateTime) result).AddMinutes ((int) minutes);
 				break;
+			default:
+				result = DBNull.Value;
+				break;
 			}
 
 			return result;
@@ -534,12 +550,10 @@ namespace Mono.Data.Tds.Protocol {
 
 			int len = (comm.GetByte() & 0xff) - 1;
 			if (len < 0)
-				return null;
+				return DBNull.Value;
 			
 			bool positive = (comm.GetByte () == 1);
 
-			if (len < 0)
-				return null;
 			if (len > 16)
 				throw new OverflowException ();
 
@@ -555,7 +569,6 @@ namespace Mono.Data.Tds.Protocol {
 		private object GetFloatValue (TdsColumnType columnType)
 		{
 			int columnSize = 0;
-			object result = null;
 
 			switch (columnType) {
 			case TdsColumnType.Real:
@@ -571,14 +584,12 @@ namespace Mono.Data.Tds.Protocol {
 
 			switch (columnSize) {
 			case 8 :
-				result = BitConverter.Int64BitsToDouble (comm.GetTdsInt64 ());
-				break;
+				return BitConverter.Int64BitsToDouble (comm.GetTdsInt64 ());
 			case 4 :
-				result = BitConverter.ToSingle (BitConverter.GetBytes (comm.GetTdsInt ()), 0);
-				break;
+				return BitConverter.ToSingle (BitConverter.GetBytes (comm.GetTdsInt ()), 0);
+			default :
+				return DBNull.Value;
 			}
-
-			return result;
 		}
 
 		private object GetImageValue ()
@@ -586,13 +597,13 @@ namespace Mono.Data.Tds.Protocol {
 			byte hasValue = comm.GetByte ();
 
 			if (hasValue == 0)
-				return null;
+				return DBNull.Value;
 			
 			comm.Skip (24);
 			int len = comm.GetTdsInt ();
 
 			if (len < 0)
-				return null;
+				return DBNull.Value;
 
 			return (comm.GetBytes (len, true));
 		}
@@ -615,7 +626,7 @@ namespace Mono.Data.Tds.Protocol {
 				len = 1; 
 				break;
 			default:
-				return null;
+				return DBNull.Value;
 			}
 
 			switch (len) {
@@ -626,7 +637,7 @@ namespace Mono.Data.Tds.Protocol {
 			case 1 :
 				return (comm.GetByte ());
 			default:
-				return null;
+				return DBNull.Value;
 			}
 		}
 
@@ -648,7 +659,7 @@ namespace Mono.Data.Tds.Protocol {
 				len = comm.GetByte ();
 				break;
 			default:
-				return null;
+				return DBNull.Value;
 			}
 
 			long rawValue = 0;
@@ -670,7 +681,7 @@ namespace Mono.Data.Tds.Protocol {
 				rawValue = BitConverter.ToInt64 (bits, 0);
 				break;
 			default:
-				return null;
+				return DBNull.Value;
 			}
 
 			result = new Decimal (rawValue);
@@ -680,24 +691,23 @@ namespace Mono.Data.Tds.Protocol {
 
 		private object GetStringValue (bool wideChars, bool outputParam)
 		{
-			object result = null;
 			bool shortLen = (tdsVersion == TdsVersion.tds70) && (wideChars || !outputParam);
-
 			int len = shortLen ? comm.GetTdsShort () : (comm.GetByte () & 0xff);
 
 			if (tdsVersion < TdsVersion.tds70 && len == 0)
-				result = null;
+				return DBNull.Value;
 			else if (len >= 0) {
+				object result;
 				if (wideChars)
 					result = comm.GetString (len / 2);
 				else
 					result = comm.GetString (len, false);
 				if (tdsVersion < TdsVersion.tds70 && ((string) result).Equals (" "))
 					result = "";
+				return result;
 			}
 			else
-				result = null;
-			return result;
+				return DBNull.Value;
 		}
 
 		protected int GetSubPacketLength ()
@@ -711,7 +721,7 @@ namespace Mono.Data.Tds.Protocol {
 			byte hasValue = comm.GetByte ();
 
 			if (hasValue != 16)
-				return null;
+				return DBNull.Value;
 
 			// 16 Byte TEXTPTR, 8 Byte TIMESTAMP
 			comm.Skip (24);
@@ -719,7 +729,7 @@ namespace Mono.Data.Tds.Protocol {
 			int len = comm.GetTdsInt ();
 
 			if (len == 0)
-				return null;
+				return DBNull.Value;
 
 			if (wideChars)
 				result = comm.GetString (len / 2);
@@ -839,6 +849,30 @@ namespace Mono.Data.Tds.Protocol {
 			}
 		}
 
+		protected internal int ProcessAuthentication ()
+		{
+			int pdu_size = Comm.GetTdsShort ();
+			byte[] msg2 = Comm.GetBytes (pdu_size, true);
+
+			Type2Message t2 = new Type2Message (msg2);
+			// 0x0001	Negotiate Unicode
+			// 0x0200	Negotiate NTLM
+			// 0x8000	Negotiate Always Sign
+
+			Type3Message t3 = new Type3Message ();
+			t3.Challenge = t2.Nonce;
+			
+			t3.Domain = this.connectionParms.DefaultDomain;
+			t3.Host = this.connectionParms.Hostname;
+			t3.Username = this.connectionParms.User;
+			t3.Password = this.connectionParms.Password;
+
+			Comm.StartPacket (TdsPacketType.SspAuth); // 0x11
+			Comm.Append (t3.GetBytes ());
+			Comm.SendPacket ();
+			return 1; // TDS_SUCCEED
+		}
+
 		protected void ProcessColumnDetail ()
 		{
 			int len = GetSubPacketLength ();
@@ -1107,6 +1141,9 @@ namespace Mono.Data.Tds.Protocol {
 			case TdsPacketSubType.LoginAck:
 				ProcessLoginAck ();
 				break;
+			case TdsPacketSubType.Authentication: // TDS 7.0
+				ProcessAuthentication ();
+				break;
 			case TdsPacketSubType.ReturnStatus :
 				Comm.Skip (4);
 				break;
