@@ -1,5 +1,5 @@
 /* Transport Security Layer (TLS)
- * Copyright (c) 2003 Carlos Guzmn lvarez
+ * Copyright (c) 2003-2004 Carlos Guzman Alvarez
  * 
  * Permission is hereby granted, free of charge, to any person 
  * obtaining a copy of this software and associated documentation 
@@ -29,7 +29,7 @@ namespace Mono.Security.Protocol.Tls
 	// Information about compression methods allowed by TLS
 	// can be found in:
 	//	draft-ietf-tls-compression-05.txt (http://www.ietf.org/internet-drafts/draft-ietf-tls-compression-05.txt)
-	public enum SecurityCompressionType : byte
+	public enum SecurityCompressionType
 	{
 		None = 0,
 		Zlib = 1
