@@ -1,5 +1,5 @@
 /* Transport Security Layer (TLS)
- * Copyright (c) 2003 Carlos Guzmn lvarez
+ * Copyright (c) 2003-2004 Carlos Guzman Alvarez
  * 
  * Permission is hereby granted, free of charge, to any person 
  * obtaining a copy of this software and associated documentation 
@@ -26,10 +26,13 @@ using System;
 
 namespace Mono.Security.Protocol.Tls
 {
-	public enum SecurityProtocolType : short
+	[Flags]
+	[Serializable]
+	public enum SecurityProtocolType
 	{
-		Default = (0x03 << 8) | 0x01,
-		Ssl3	= (0x03 << 8) | 0x00,
-		Tls		= (0x03 << 8) | 0x01
+		Default = -1073741824,
+		Ssl2	= 12,
+		Ssl3	= 48,
+		Tls		= 192
 	}
 }
\ No newline at end of file
