@@ -35,14 +35,14 @@ namespace NpgsqlTests
 	public class CommandTests
 	{
 		private NpgsqlConnection 	_conn = null;
-		private String 						_connString = "Server=localhost;User ID=npgsql_tests;Password=npgsql_tests;Database=npgsql_tests";
+		private String 						_connString = "Server=localhost;User ID=npgsql_tests;Password=npgsql_tests;Database=npgsql_tests;SSL=yes";
 		
 		[SetUp]
 		protected void SetUp()
 		{
-			NpgsqlEventLog.Level = LogLevel.None;
-			//NpgsqlEventLog.Level = LogLevel.Debug;
-			//NpgsqlEventLog.LogName = "NpgsqlTests.LogFile";
+			//NpgsqlEventLog.Level = LogLevel.None;
+			NpgsqlEventLog.Level = LogLevel.Debug;
+			NpgsqlEventLog.LogName = "NpgsqlTests.LogFile";
 			_conn = new NpgsqlConnection(_connString);
 		}
 		
@@ -321,7 +321,7 @@ namespace NpgsqlTests
 		  NpgsqlCommand command = new NpgsqlCommand("listen notifytest;", _conn);
 		  command.ExecuteNonQuery();
 		  
-		  _conn.OnNotification += new NotificationEventHandler(NotificationSupportHelper);
+		  _conn.Notification += new NotificationEventHandler(NotificationSupportHelper);
 		  
 		                                                       
 		  command = new NpgsqlCommand("notify notifytest;", _conn);
@@ -414,6 +414,72 @@ namespace NpgsqlTests
 			
 			
 			
+		}
+		
+		[Test]
+		public void InsertSingleValue()
+		{
+		  _conn.Open();
+			
+			
+			NpgsqlCommand command = new NpgsqlCommand("insert into tabled(field_float4) values (:a)", _conn);
+			command.Parameters.Add(new NpgsqlParameter(":a", DbType.Single));
+			
+			command.Parameters[0].Value = 7.4F;
+			
+			Int32 rowsAdded = command.ExecuteNonQuery();
+			
+			Assertion.AssertEquals(1, rowsAdded);
+			
+			command.CommandText = "select * from tabled where field_float4 = :a";
+			
+			
+			NpgsqlDataReader dr = command.ExecuteReader();
+			dr.Read();
+			
+			Single result = dr.GetFloat(1);
+			
+			
+			command.CommandText = "delete from tabled where field_serial > 2;";
+			command.Parameters.Clear();
+			command.ExecuteNonQuery();
+			
+			
+			Assertion.AssertEquals(7.4F, result);
+			
+		}
+		
+		[Test]
+		public void InsertDoubleValue()
+		{
+		  _conn.Open();
+			
+			
+			NpgsqlCommand command = new NpgsqlCommand("insert into tabled(field_float8) values (:a)", _conn);
+			command.Parameters.Add(new NpgsqlParameter(":a", DbType.Double));
+			
+			command.Parameters[0].Value = 7.4D;
+			
+			Int32 rowsAdded = command.ExecuteNonQuery();
+			
+			Assertion.AssertEquals(1, rowsAdded);
+			
+			command.CommandText = "select * from tabled where field_float8 = :a";
+			
+			
+			NpgsqlDataReader dr = command.ExecuteReader();
+			dr.Read();
+			
+			Double result = dr.GetDouble(2);
+			
+			
+			command.CommandText = "delete from tabled where field_serial > 2;";
+			command.Parameters.Clear();
+			//command.ExecuteNonQuery();
+			
+			
+			Assertion.AssertEquals(7.4D, result);
+			
 		}
 		
 		[Test]
@@ -627,9 +693,123 @@ namespace NpgsqlTests
 			Assertion.AssertEquals(5, result);
 			
 		}
+        
+        [Test]
+		public void AnsiStringSupport()
+		{
+			_conn.Open();
+			
+			NpgsqlCommand command = new NpgsqlCommand("insert into tablea(field_text) values (:a)", _conn);
+			
+			command.Parameters.Add(new NpgsqlParameter("a", DbType.AnsiString));
+			
+			command.Parameters[0].Value = "TesteAnsiString";
+			
+			Int32 rowsAdded = command.ExecuteNonQuery();
+			
+			Assertion.AssertEquals(1, rowsAdded);
+			
+			command.CommandText = String.Format("select count(*) from tablea where field_text = '{0}'", command.Parameters[0].Value);
+			command.Parameters.Clear();
+			
+			Object result = command.ExecuteScalar(); // The missed cast is needed as Server7.2 returns Int32 and Server7.3+ returns Int64
+			
+			command.CommandText = "delete from tablea where field_serial = (select max(field_serial) from tablea);";
+			command.ExecuteNonQuery();
+			
+			Assertion.AssertEquals(1, result);
+			
+		}
 		
 		
-		
+        [Test]
+		public void MultipleQueriesFirstResultsetEmpty()
+		{
+			_conn.Open();
+			
+			NpgsqlCommand command = new NpgsqlCommand("insert into tablea(field_text) values ('a'); select count(*) from tablea;", _conn);
+            
+            Object result = command.ExecuteScalar();
+                        
+            
+            command.CommandText = "delete from tablea where field_serial > 5";
+            command.ExecuteNonQuery();
+            
+            command.CommandText = "select * from tablea where field_serial = 0";
+            command.ExecuteScalar();
+            
+            
+            Assertion.AssertEquals(6, result);
+            
+            
+		}
+        
+        [Test]
+        [ExpectedException(typeof(NpgsqlException))]
+        public void ConnectionStringWithInvalidParameters()
+        {
+            NpgsqlConnection conn = new NpgsqlConnection("Server=127.0.0.1;User Id=npgsql_tests;Password=j");
+            
+            NpgsqlCommand command = new NpgsqlCommand("select * from tablea", conn);
+            
+            command.Connection.Open();
+            command.ExecuteReader();
+            command.Connection.Close();
+            
+            
+        }
+        
+		[Test]
+        [ExpectedException(typeof(NpgsqlException))]
+        public void InvalidConnectionString()
+        {
+            NpgsqlConnection conn = new NpgsqlConnection("Server=127.0.0.1;User Id=npgsql_tests");
+            
+            NpgsqlCommand command = new NpgsqlCommand("select * from tablea", conn);
+            
+            command.Connection.Open();
+            command.ExecuteReader();
+            command.Connection.Close();
+            
+            
+        }
+        
+        [Test]
+        public void AmbiguousFunctionParameterType()
+        {
+            NpgsqlConnection conn = new NpgsqlConnection("Server=127.0.0.1;User Id=npgsql_tests;Password=npgsql_tests");
+            
+            
+            NpgsqlCommand command = new NpgsqlCommand("ambiguousParameterType(:a, :b, :c, :d, :e, :f)", conn);
+            command.CommandType = CommandType.StoredProcedure;
+            NpgsqlParameter p = new NpgsqlParameter("a", DbType.Int16);
+            p.Value = 2;
+            command.Parameters.Add(p);
+            p = new NpgsqlParameter("b", DbType.Int32);
+            p.Value = 2;
+            command.Parameters.Add(p);
+            p = new NpgsqlParameter("c", DbType.Int64);
+            p.Value = 2;
+            command.Parameters.Add(p);
+            p = new NpgsqlParameter("d", DbType.String);
+            p.Value = "a";
+            command.Parameters.Add(p);
+            p = new NpgsqlParameter("e", DbType.String);
+            p.Value = "a";
+            command.Parameters.Add(p);
+            p = new NpgsqlParameter("f", DbType.String);
+            p.Value = "a";
+            command.Parameters.Add(p);
+            
+            
+            command.Connection.Open();
+            command.Prepare();
+            command.ExecuteScalar();
+            command.Connection.Close();
+            
+            
+        }
+        
 		
 	}
 }
