@@ -7,7 +7,7 @@
 // Copyright (C) Tim Coleman, 2003
 //
 
-#if NET_1_2
+#if NET_2_0
 
 using System.Data.SqlXml;
 
@@ -22,7 +22,7 @@ namespace System.Data.Mapping {
 		}
 
 		[MonoTODO]
-		public string name { 
+		public string Name { 
 			get { throw new NotImplementedException (); }
 		}
 
@@ -65,4 +65,4 @@ namespace System.Data.Mapping {
         }
 }
 
-#endif // NET_1_2
+#endif // NET_2_0
