@@ -19,8 +19,10 @@ using System.Runtime.InteropServices;
 #if (NET_1_0)
 	[assembly: AssemblyVersion("1.0.3300.0")]
 	[assembly: SatelliteContractVersion("1.0.3300.0")]
-#endif
-#if (NET_1_1)
+#elif (NET_2_0)
+        [assembly: AssemblyVersion("2.0.3600.0")]
+	[assembly: SatelliteContractVersion("2.0.3600.0")]
+#elif (NET_1_1)
 	[assembly: AssemblyVersion("1.0.5000.0")]
 	[assembly: SatelliteContractVersion("1.0.5000.0")]
 	[assembly: ComCompatibleVersion(1, 0, 3300, 0)]
@@ -35,8 +37,7 @@ using System.Runtime.InteropServices;
 [assembly: AssemblyCopyright("(c) 2003 Various Authors")]
 [assembly: AssemblyTrademark("")]
 
-// FIXME Add once compliance is reached
-//[assembly: CLSCompliant(true)]
+[assembly: CLSCompliant(true)]
 [assembly: AssemblyDefaultAlias("System.Data.dll")]
 [assembly: AssemblyInformationalVersion("0.0.0.1")]
 [assembly: NeutralResourcesLanguage("en-US")]
@@ -44,5 +45,5 @@ using System.Runtime.InteropServices;
 [assembly: ComVisible(false)]
 [assembly: AllowPartiallyTrustedCallers]
 
-//[assembly: AssemblyDelaySign(false)]
-//[assembly: AssemblyKeyFile("")]
\ No newline at end of file
+[assembly: AssemblyDelaySign(true)]
+[assembly: AssemblyKeyFile("../ecma.pub")]
