@@ -6,7 +6,7 @@
 //   Tim Coleman (tim@timcoleman.com)
 //
 // (C) Ximian, Inc
-// Copyright (C) Tim Coleman, 2002
+// Copyright (C) Tim Coleman, 2002-2003
 //
 
 using System;
@@ -60,7 +60,7 @@ namespace System.Data.Common {
 		public DataColumnMapping this [string sourceColumn] {
 			get {
 				if (!Contains(sourceColumn)) {
-					throw new IndexOutOfRangeException("DataColumnMappingCollection doesn't contains DataColumnMapping with SourceColumn 'field_int2'.");
+					throw new IndexOutOfRangeException("DataColumnMappingCollection doesn't contains DataColumnMapping with SourceColumn '" + sourceColumn + "'.");
 				}
 				return (DataColumnMapping) sourceColumns[sourceColumn]; }
 			set { this [list.IndexOf (sourceColumns[sourceColumn])] = value; }
@@ -122,6 +122,14 @@ namespace System.Data.Common {
 			return mapping;
 		}
 
+#if NET_2_0
+		[MonoTODO]
+		public void AddRange (Array values)
+		{
+			throw new NotImplementedException ();
+		}
+#endif
+
 		public void AddRange (DataColumnMapping[] values) 
 		{
 			foreach (DataColumnMapping mapping in values)
@@ -165,6 +173,14 @@ namespace System.Data.Common {
 			return new DataColumnMapping (sourceColumn, sourceColumn);
 		}
 
+#if NET_2_0
+		[MonoTODO]
+		public static DataColumn GetDataColumn (DataColumnMappingCollection columnMappings, string sourceColumn, Type dataType, DataTable dataTable, MissingMappingAction mappingAction, MissingSchemaAction schemaAction)
+		{
+			throw new NotImplementedException ();
+		}
+#endif
+
 		public IEnumerator GetEnumerator ()
 		{
 			return list.GetEnumerator ();
