@@ -6,13 +6,14 @@
 //   Tim Coleman (tim@timcoleman.com)
 //
 // (C) Ximian, Inc
-// Copyright (C) 2002 Tim Coleman
+// Copyright (C) Tim Coleman, 2002-2003
 //
 
 using System;
 using System.Collections;
 using System.ComponentModel;
 using System.Data;
+using System.Runtime.InteropServices;
 
 namespace System.Data.Common {
 	public abstract class DbDataAdapter : DataAdapter, ICloneable
@@ -34,18 +35,63 @@ namespace System.Data.Common {
 
 		#region Properties
 
+#if NET_2_0
+		[MonoTODO]
+		protected virtual IDbConnection BaseConnection {
+			get { throw new NotImplementedException (); }
+			set { throw new NotImplementedException (); }
+		}
+
+		public IDbConnection Connection { 
+			get { return BaseConnection; }
+			set { BaseConnection = value; }
+		}
+#endif
+
 		IDbCommand DeleteCommand {
 			get { return ((IDbDataAdapter) this).DeleteCommand; }
 		}
 
+#if NET_2_0
+		protected internal CommandBehavior FillCommandBehavior {
+			get { throw new NotImplementedException (); }
+			set { throw new NotImplementedException (); }
+		}
+#endif
+
 		IDbCommand InsertCommand {
 			get { return ((IDbDataAdapter) this).InsertCommand; }
 		}
 
+#if NET_2_0
+		[MonoTODO]
+		protected virtual IDbCommand this [[Optional] StatementType statementType] {
+			get { throw new NotImplementedException (); }
+			set { throw new NotImplementedException (); }
+		}
+
+		protected virtual DbProviderFactory ProviderFactory {
+			get { throw new NotImplementedException (); }
+		}
+#endif
+
 		IDbCommand SelectCommand {
 			get { return ((IDbDataAdapter) this).SelectCommand; }
 		}
 
+#if NET_2_0
+		[MonoTODO]
+		public IDbTransaction Transaction {
+			get { throw new NotImplementedException (); }
+			set { throw new NotImplementedException (); }
+		}
+
+		[MonoTODO]
+		public int UpdateBatchSize {
+			get { throw new NotImplementedException (); }
+			set { throw new NotImplementedException (); }
+		}
+#endif
 
 		IDbCommand UpdateCommand {
 			get { return ((IDbDataAdapter) this).UpdateCommand; }
@@ -55,14 +101,25 @@ namespace System.Data.Common {
 		
 		#region Events
 
+#if ONLY_1_0 || ONLY_1_1
+
 		[DataCategory ("Fill")]
 		[DataSysDescription ("Event triggered when a recoverable error occurs during Fill.")]
 		public event FillErrorEventHandler FillError;
 
+#endif
 		#endregion // Events
 
 		#region Methods
 
+#if NET_2_0
+		[MonoTODO]
+		public virtual void BeginInit ()
+		{
+			throw new NotImplementedException ();
+		}
+#endif
+
 		protected abstract RowUpdatedEventArgs CreateRowUpdatedEvent (DataRow dataRow, IDbCommand command, StatementType statementType, DataTableMapping tableMapping);
 		protected abstract RowUpdatingEventArgs CreateRowUpdatingEvent (DataRow dataRow, IDbCommand command, StatementType statementType, DataTableMapping tableMapping);
 
@@ -97,6 +154,14 @@ namespace System.Data.Common {
 			}
 		}
 
+#if NET_2_0
+		[MonoTODO]
+		public virtual void EndInit ()
+		{
+			throw new NotImplementedException ();
+		}
+#endif
+
 		public override int Fill (DataSet dataSet)
 		{
 			return Fill (dataSet, 0, 0, DefaultSourceTableName, SelectCommand, CommandBehavior.Default);
@@ -115,53 +180,26 @@ namespace System.Data.Common {
 			return Fill (dataSet, 0, 0, srcTable, SelectCommand, CommandBehavior.Default);
 		}
 
+#if NET_2_0
+		protected override int Fill (DataTable dataTable, IDataReader dataReader) 
+#else
 		protected virtual int Fill (DataTable dataTable, IDataReader dataReader) 
+#endif
 		{
-			int count = 0;
-			bool doContinue = true;
-
 			if (dataReader.FieldCount == 0) {
 				dataReader.Close ();
 				return 0;
 			}
 			
-			try
-			{
-				object[] itemArray = new object [dataReader.FieldCount];
+			int count = 0;
+
+			try {
 				string tableName = SetupSchema (SchemaType.Mapped, dataTable.TableName);
-				if (tableName != null)
-				{
+				if (tableName != null) {
 					dataTable.TableName = tableName;
-					Hashtable mapping = BuildSchema (dataReader, dataTable, SchemaType.Mapped);
-
-					while (doContinue && dataReader.Read ()) 
-					{
-						// we get the values from the datareader
-						dataReader.GetValues (itemArray);
-
-						// we only need the values that has a mapping to the table.
-						object[] tableArray = new object[mapping.Count];
-						for (int i = 0; i < tableArray.Length; i++)
-							tableArray[i] = mapping[i]; // get the value for each column
-
-						try 
-						{
-							dataTable.BeginLoadData ();
-							dataTable.LoadDataRow (itemArray, AcceptChangesDuringFill);
-							dataTable.EndLoadData ();
-							count += 1;
-						}
-						catch (Exception e) 
-						{
-							FillErrorEventArgs args = CreateFillErrorEvent (dataTable, itemArray, e);
-							OnFillError (args);
-							doContinue = args.Continue;
-						}
-					}
+					FillTable (dataTable, dataReader, 0, 0, ref count);
 				}
-			}
-			finally
-			{
+			} finally {
 				dataReader.Close ();
 			}
 
@@ -180,78 +218,58 @@ namespace System.Data.Common {
 			return Fill (dataTable, command.ExecuteReader (commandBehavior));
 		}
 
+#if NET_2_0
+		[MonoTODO]
+		public int Fill (int startRecord, int maxRecords, DataTable[] dataTables)
+		{
+			throw new NotImplementedException ();
+		}
+#endif
+
 		public int Fill (DataSet dataSet, int startRecord, int maxRecords, string srcTable) 
 		{
 			return this.Fill (dataSet, startRecord, maxRecords, srcTable, SelectCommand, CommandBehavior.Default);
 		}
 
+#if NET_2_0
+		[MonoTODO]
+		protected virtual int Fill (DataTable[] dataTables, int startRecord, int maxRecords, IDbCommand command, CommandBehavior behavior)
+		{
+			throw new NotImplementedException ();
+		}
+#endif
+
+#if NET_2_0
+		protected override int Fill (DataSet dataSet, string srcTable, IDataReader dataReader, int startRecord, int maxRecords) 
+#else
 		protected virtual int Fill (DataSet dataSet, string srcTable, IDataReader dataReader, int startRecord, int maxRecords) 
+#endif
 		{
 			if (startRecord < 0)
 				throw new ArgumentException ("The startRecord parameter was less than 0.");
 			if (maxRecords < 0)
 				throw new ArgumentException ("The maxRecords parameter was less than 0.");
 
-			if (dataReader.FieldCount == 0) {
-				dataReader.Close ();
-				return 0;
-			}
-
-                        DataTable dataTable;
-                        int resultIndex = 0;
-                        int count = 0;
-			bool doContinue = true;
+			DataTable dataTable;
+			int resultIndex = 0;
+			int count = 0;
 			
-			try
-			{
+			try {
 				string tableName = srcTable;
-				object[] itemArray;
-
-				do 
-				{
+				do {
 					tableName = SetupSchema (SchemaType.Mapped, tableName);
-					if (tableName != null)
-					{
+					if (tableName != null) {
 						// check if the table exists in the dataset
 						if (dataSet.Tables.Contains (tableName)) 
 							// get the table from the dataset
 							dataTable = dataSet.Tables [tableName];
-						else
-						{
+						else {
 							dataTable = new DataTable(tableName);
 							dataSet.Tables.Add (dataTable);
 						}
-						Hashtable mapping = BuildSchema (dataReader, dataTable, SchemaType.Mapped);
 
-						for (int i = 0; i < startRecord; i += 1)
-							dataReader.Read ();
-						
-						itemArray = new object [dataReader.FieldCount];
-
-						while (doContinue && dataReader.Read () && !(maxRecords > 0 && count >= maxRecords)) 
-						{
-							// we get the values from the datareader
-							dataReader.GetValues (itemArray);
-							
-							// we only need the values that has a mapping to the table.
-							object[] tableArray = new object[mapping.Count];
-							for (int i = 0; i < tableArray.Length; i++)
-								tableArray[i] = itemArray[(int)mapping[i]]; // get the value for each column
-							
-							try 
-							{
-								dataTable.BeginLoadData ();
-								//dataTable.LoadDataRow (itemArray, AcceptChangesDuringFill);
-								dataTable.LoadDataRow (tableArray, AcceptChangesDuringFill);
-								dataTable.EndLoadData ();
-								count += 1;
-							}
-							catch (Exception e) 
-							{
-								FillErrorEventArgs args = CreateFillErrorEvent (dataTable, itemArray, e);
-								OnFillError (args);
-								doContinue = args.Continue;
-							}
+						if (!FillTable (dataTable, dataReader, startRecord, maxRecords, ref count)) {
+							continue;
 						}
 
 						tableName = String.Format ("{0}{1}", srcTable, ++resultIndex);
@@ -259,20 +277,19 @@ namespace System.Data.Common {
 						startRecord = 0;
 						maxRecords = 0;
 					}
-
-				} while (doContinue && dataReader.NextResult ());
-			}
-			finally
-			{
+				} while (dataReader.NextResult ());
+			} 
+			finally {
 				dataReader.Close ();
 			}
 
                         return count;
 		}
-
 		
 		protected virtual int Fill (DataSet dataSet, int startRecord, int maxRecords, string srcTable, IDbCommand command, CommandBehavior behavior) 
 		{
+			if (MissingSchemaAction == MissingSchemaAction.AddWithKey)
+			    behavior |= CommandBehavior.KeyInfo;
 			CommandBehavior commandBehavior = behavior;
 			if (command.Connection.State == ConnectionState.Closed) {
 				command.Connection.Open ();
@@ -281,6 +298,46 @@ namespace System.Data.Common {
 			return Fill (dataSet, srcTable, command.ExecuteReader (commandBehavior), startRecord, maxRecords);
 		}
 
+		private bool FillTable (DataTable dataTable, IDataReader dataReader, int startRecord, int maxRecords, ref int counter) 
+		{
+			if (dataReader.FieldCount == 0) {
+				return false;
+			}
+
+			int counterStart = counter;
+			
+			int[] mapping = BuildSchema (dataReader, dataTable, SchemaType.Mapped);
+
+				for (int i = 0; i < startRecord; i++) {
+					dataReader.Read ();
+				}
+
+			while (dataReader.Read () && (maxRecords == 0 || (counter - counterStart) < maxRecords)) {
+				try {
+					dataTable.BeginLoadData ();
+					dataTable.LoadDataRow (dataReader, mapping, AcceptChangesDuringFill);
+					dataTable.EndLoadData ();
+					counter++;
+				} 
+				catch (Exception e) {
+					object[] readerArray = new object[dataReader.FieldCount];
+					object[] tableArray = new object[mapping.Length];
+					// we get the values from the datareader
+					dataReader.GetValues (readerArray);
+					// copy from datareader columns to table columns according to given mapping
+					for (int i = 0; i < mapping.Length; i++) {
+						tableArray[i] = readerArray[mapping[i]];
+					}
+					FillErrorEventArgs args = CreateFillErrorEvent (dataTable, tableArray, e);
+					OnFillError (args);
+					if(!args.Continue) {
+						return false;
+					}
+				}
+			}
+			return true;
+		}
+
 		public override DataTable[] FillSchema (DataSet dataSet, SchemaType schemaType) 
 		{
 			return FillSchema (dataSet, schemaType, SelectCommand, DefaultSourceTableName, CommandBehavior.Default);
@@ -359,6 +416,20 @@ namespace System.Data.Common {
 			return (DataTable[]) output.ToArray (typeof (DataTable));
 		}
 
+#if NET_2_0
+		[MonoTODO]
+		public DataSet GetDataSet ()
+		{
+			throw new NotImplementedException ();
+		}
+
+		[MonoTODO]
+		public DataTable GetDataTable ()
+		{
+			throw new NotImplementedException ();
+		}
+#endif
+
 		private string SetupSchema (SchemaType schemaType, string sourceTableName)
 		{
 			DataTableMapping tableMapping = null;
@@ -383,23 +454,22 @@ namespace System.Data.Common {
 			return parameters;
 		}
 		
-		// this method bulds the schema for a given datatable
-		// returns a hashtable that his keys are the ordinal of the datatable columns, and his values
-		// are the indexes of the source columns in the data reader.
-		// each column in the datatable has a mapping to a specific column in the datareader
-		// the hashtable represents this match.
+		// this method builds the schema for a given datatable. it returns a int array with 
+		// "array[ordinal of datatable column] == index of source column in data reader".
+		// each column in the datatable has a mapping to a specific column in the datareader,
+		// the int array represents this match.
 		[MonoTODO ("Test")]
-		private Hashtable BuildSchema (IDataReader reader, DataTable table, SchemaType schemaType)
+		private int[] BuildSchema (IDataReader reader, DataTable table, SchemaType schemaType)
 		{
 			int readerIndex = 0;
-			Hashtable mapping = new Hashtable(); // hashing the reader indexes with the datatable indexes
+			int[] mapping = new int[reader.FieldCount]; // mapping the reader indexes to the datatable indexes
 			ArrayList primaryKey = new ArrayList ();
 			ArrayList sourceColumns = new ArrayList ();
 
 			foreach (DataRow schemaRow in reader.GetSchemaTable ().Rows) {
 				// generate a unique column name in the source table.
 				string sourceColumnName;
-				if (schemaRow ["ColumnName"].Equals (DBNull.Value))
+				if (schemaRow.IsNull("ColumnName"))
 					sourceColumnName = DefaultSourceColumnName;
 				else 
 					sourceColumnName = (string) schemaRow ["ColumnName"];
@@ -413,8 +483,7 @@ namespace System.Data.Common {
 				// generate DataSetColumnName from DataTableMapping, if any
 				string dsColumnName = realSourceColumnName;
 				DataTableMapping tableMapping = null;
-				if (schemaType == SchemaType.Mapped)
-					tableMapping = DataTableMappingCollection.GetTableMappingBySchemaAction (TableMappings, table.TableName, table.TableName, MissingMappingAction); 
+				tableMapping = DataTableMappingCollection.GetTableMappingBySchemaAction (TableMappings, table.TableName, table.TableName, MissingMappingAction); 
 				if (tableMapping != null) 
 				{
 					
@@ -443,13 +512,13 @@ namespace System.Data.Common {
 									primaryKey.Add (col);
 							
 							// add the ordinal of the column as a key and the index of the column in the datareader as a value.
-							mapping.Add(col.Ordinal, readerIndex);
+							mapping[col.Ordinal] = readerIndex;
 						}
 					}
 				}
 				readerIndex++;
 			}
-			if (MissingSchemaAction == MissingSchemaAction.AddWithKey && primaryKey.Count > 0)
+			if (primaryKey.Count > 0)
 				table.PrimaryKey = (DataColumn[])(primaryKey.ToArray(typeof (DataColumn)));
 
 			return mapping;
@@ -594,20 +663,28 @@ namespace System.Data.Common {
 
 						parameter.Value = row [dsColumnName, rowVersion];
 					}
-					row.AcceptChanges ();
 				}
-
-				if (command.Connection.State == ConnectionState.Closed) 
-					command.Connection.Open ();
 				
+				CommandBehavior commandBehavior = CommandBehavior.Default;
+				if (command.Connection.State == ConnectionState.Closed) 
+				{
+					command.Connection.Open ();
+					commandBehavior |= CommandBehavior.CloseConnection;
+				}
+				
+				IDataReader reader = null;
 				try
 				{
-					int tmp = command.ExecuteNonQuery ();
+					// use ExecuteReader because we want to use the commandbehavior parameter.
+					// so the connection will be closed if needed.
+					reader = command.ExecuteReader (commandBehavior);
+					int tmp = reader.RecordsAffected;
 					// if the execute does not effect any rows we throw an exception.
 					if (tmp == 0)
 						throw new DBConcurrencyException("Concurrency violation: the " + commandName +"Command affected 0 records.");
 					updateCount += tmp;
 					OnRowUpdated (CreateRowUpdatedEvent (row, command, statementType, tableMapping));
+					row.AcceptChanges ();
 				}
 				catch (Exception e)
 				{
@@ -616,6 +693,11 @@ namespace System.Data.Common {
 					else
 						throw e;
 				}
+				finally
+				{
+					if (reader != null)
+						reader.Close ();
+				}
 			}
 			
 			return updateCount;
@@ -635,11 +717,13 @@ namespace System.Data.Common {
 			return Update (dataTable, tableMapping);
 		}
 
+#if ONLY_1_0 || ONLY_1_1
 		protected virtual void OnFillError (FillErrorEventArgs value) 
 		{
 			if (FillError != null)
 				FillError (this, value);
 		}
+#endif
 
 		protected abstract void OnRowUpdated (RowUpdatedEventArgs value);
 		protected abstract void OnRowUpdating (RowUpdatingEventArgs value);
