@@ -35,7 +35,6 @@ namespace System.Data.Odbc
 		{
 			this.command = command;
 			this.behavior=behavior;
-			this.command.Connection.DataReader = this;
 			open = true;
 			currentRow = -1;
 			hstmt=command.hStmt;
@@ -95,6 +94,11 @@ namespace System.Data.Odbc
 			}
 		}
 
+		[MonoTODO]
+		public bool HasRows {
+			get { throw new NotImplementedException(); }
+		}
+
 		#endregion
 
 		#region Methods
@@ -150,8 +154,6 @@ namespace System.Data.Odbc
 			open = false;
 			currentRow = -1;
 
-			this.command.Connection.DataReader = null;
-
 			if ((behavior & CommandBehavior.CloseConnection)==CommandBehavior.CloseConnection)
 				this.command.Connection.Close();
 		}
@@ -192,6 +194,7 @@ namespace System.Data.Odbc
 		}
 
 		[MonoTODO]
+		 [EditorBrowsableAttribute (EditorBrowsableState.Never)]
 		public OdbcDataReader GetData (int ordinal)
 		{
 			throw new NotImplementedException ();
@@ -202,6 +205,10 @@ namespace System.Data.Odbc
 			return GetColumn(index).OdbcType.ToString();
 		}
 
+		public DateTime GetDate(int ordinal) {
+			return GetDateTime(ordinal);
+		}
+
 		public DateTime GetDateTime (int ordinal)
 		{
 			return (DateTime) GetValue(ordinal);
@@ -365,7 +372,7 @@ namespace System.Data.Odbc
 		}
 
 		[MonoTODO]
-		public TimeSpan GetTimeSpan (int ordinal)
+		public TimeSpan GetTime (int ordinal)
 		{
 			throw new NotImplementedException ();
 		}
