@@ -22,6 +22,13 @@ namespace System.Data.Odbc
 	
 		#endregion // Fields
 
+		#region Constructors
+
+		internal OdbcErrorCollection() {
+		}
+
+		#endregion Constructors
+
 		#region Properties 
 
 		public int Count 
@@ -60,16 +67,25 @@ namespace System.Data.Odbc
 
 		#region Methods
 
-		public void Add (OdbcError error)
+		internal void Add (OdbcError error)
 		{
 			list.Add ((object) error);
 		}
 		
-		[MonoTODO]
-		public void CopyTo (Array array, int index) 
+		public void CopyTo (Array array, int index)
 		{
+			if (array == null)
+				throw new ArgumentNullException("array");		
+			
+			if ((index < array.GetLowerBound (0)) || (index > array.GetUpperBound (0)))
+				throw new ArgumentOutOfRangeException("index");
+		
+			// is the check for IsFixedSize required?
+			if ((array.IsFixedSize) || (index + this.Count > array.GetUpperBound (0)))
+				throw new ArgumentException("array");
+
 			((OdbcError[])(list.ToArray ())).CopyTo (array, index);
-			throw new NotImplementedException ();
+			 
 		}
 
 		public IEnumerator GetEnumerator ()
