@@ -10,9 +10,11 @@
 using System;
 using System.Data;
 using System.Data.Common;
+using System.ComponentModel;
 
 namespace System.Data.Odbc
 {
+	[TypeConverterAttribute (typeof (OdbcParameterConverter))]	
 	public sealed class OdbcParameter : MarshalByRefObject, IDbDataParameter, IDataParameter, ICloneable
 	{
 		#region Fields
@@ -28,7 +30,8 @@ namespace System.Data.Odbc
 		ParameterDirection direction;
 		OdbcType odbcType;
 		DbType dbType;
-
+		OdbcParameterCollection container = null;	
+		
 		// Buffers for parameter value based on type. Currently I've only optimized 
 		// for int parameters and everything else is just converted to a string.
 		private bool bufferIsSet;
@@ -91,6 +94,18 @@ namespace System.Data.Odbc
 
 		#region Properties
 
+		// Used to ensure that only one collection can contain this
+                // parameter
+                internal OdbcParameterCollection Container {
+                        get { return container; }
+                        set { container = value; }
+                }
+		
+		[BrowsableAttribute (false)]
+                [OdbcDescriptionAttribute ("The parameter generic type")]
+                [RefreshPropertiesAttribute (RefreshProperties.All)]
+                [DesignerSerializationVisibilityAttribute (DesignerSerializationVisibility.Hidden)]
+		[OdbcCategory ("Data")]
 		public DbType DbType {
 			get { return dbType; }
 			set { 
@@ -98,15 +113,28 @@ namespace System.Data.Odbc
 			}
 		}
 		
+		[OdbcCategory ("Data")]
+		[OdbcDescriptionAttribute ("Input, output, or bidirectional parameter")]  
+		[DefaultValue (ParameterDirection.Input)]
 		public ParameterDirection Direction {
 			get { return direction; }
 			set { direction = value; }
 		}
 		
+		[BrowsableAttribute (false)]
+                [OdbcDescriptionAttribute ("A design-time property used for strongly typed code generation")]
+                [DesignOnlyAttribute (true)]
+                [EditorBrowsableAttribute (EditorBrowsableState.Advanced)]
+                [DefaultValue (false)]
 		public bool IsNullable {
 			get { return isNullable; }
+			set { isNullable = value; }
 		}
 
+		[DefaultValue (OdbcType.NChar)]
+                [OdbcDescriptionAttribute ("The parameter native type")]
+                [RefreshPropertiesAttribute (RefreshProperties.All)]
+		[OdbcCategory ("Data")]
 		public OdbcType OdbcType {
 			get { return odbcType; }
 			set {
@@ -156,54 +184,48 @@ namespace System.Data.Odbc
 
 		#endregion // Properties
 
-		#region public Properties
+		#region Methods
 
-		public void Bind(IntPtr hstmt,int ParamNum)
-		{
+		internal void Bind(IntPtr hstmt, int ParamNum) {
 			OdbcReturn ret;
 			// Set up the buffer if we haven't done so yet
-			if(!bufferIsSet)
+			if (!bufferIsSet)
 				setBuffer();
-				
+
 			// Convert System.Data.ParameterDirection into odbc enum
-			OdbcInputOutputDirection paramdir=libodbc.ConvertParameterDirection(this.direction);
+			OdbcInputOutputDirection paramdir = libodbc.ConvertParameterDirection(this.direction);
 			// Bind parameter based on type
-			if (odbcType==OdbcType.Int)
-				ret=libodbc.SQLBindParameter(hstmt, (ushort) ParamNum, (short) paramdir, 
-					(short) odbcType, (short) odbcType, Convert.ToUInt32(size), 
+			if (odbcType == OdbcType.Int)
+				ret = libodbc.SQLBindParameter(hstmt, (ushort)ParamNum, (short)paramdir,
+					(short)odbcType, (short)odbcType, Convert.ToUInt32(size),
 					0, ref intbuf, 0, 0);
 			else
-				ret=libodbc.SQLBindParameter(hstmt, (ushort) ParamNum, 	(short) paramdir,
-					(short) OdbcType.Char, (short) odbcType, Convert.ToUInt32(size), 
-					0, 	buffer, 0, 0);
+				ret = libodbc.SQLBindParameter(hstmt, (ushort)ParamNum, (short)paramdir,
+					(short)OdbcType.Char, (short)odbcType, Convert.ToUInt32(size),
+					0, buffer, 0, 0);
 			// Check for error condition
-			if ((ret!=OdbcReturn.Success) && (ret!=OdbcReturn.SuccessWithInfo)) 
-				throw new OdbcException(new OdbcError("SQLBindParam",OdbcHandleType.Stmt,hstmt));
+			if ((ret != OdbcReturn.Success) && (ret != OdbcReturn.SuccessWithInfo))
+				throw new OdbcException(new OdbcError("SQLBindParam", OdbcHandleType.Stmt, hstmt));
 		}
 
 		private void setBuffer() {
 			// Load buffer with new value
-			if (odbcType==OdbcType.Int)
-				intbuf=(int) ParamValue;
-			else
-			{
+			if (odbcType == OdbcType.Int)
+				intbuf = (int)ParamValue;
+			else {
 				string paramValueString = ParamValue.ToString();
 				// Treat everything else as a string
 				// Init string buffer
-				if (buffer==null || buffer.Length< ((size>20)?size:20) )
-					buffer=new byte[(size>20)?size:20];
+				if (buffer == null || buffer.Length < ((size > 20) ? size : 20))
+					buffer = new byte[(size > 20) ? size : 20];
 				else
 					buffer.Initialize();
 				// Convert value into string and store into buffer
-				System.Text.Encoding.ASCII.GetBytes(paramValueString, 0, paramValueString.Length , buffer, 0);
+				System.Text.Encoding.ASCII.GetBytes(paramValueString, 0, paramValueString.Length, buffer, 0);
 			}
 			bufferIsSet = true;
 		}
 
-		#endregion // public Properties
-
-		#region Methods
-
 		[MonoTODO]
 		object ICloneable.Clone ()
 		{
