@@ -114,6 +114,13 @@ namespace System.Data.OleDb
 				return FieldCount > 0 ? -1 : total_rows;
 			}
 		}
+		
+		[MonoTODO]
+		public bool HasRows {
+			get {
+				throw new NotImplementedException ();
+			}
+		}
 
 		#endregion
 
@@ -174,6 +181,7 @@ namespace System.Data.OleDb
 			throw new NotImplementedException ();
 		}
 		
+		[EditorBrowsableAttribute (EditorBrowsableState.Never)]
 		public char GetChar (int ordinal)
 		{
 			IntPtr value;
@@ -427,7 +435,9 @@ namespace System.Data.OleDb
 
 				if (currentResult == -1) {
 					// FIXME: throw an exception?
+#if DEBUG_OleDbDataReader
 					Console.WriteLine("Error: current result -1");
+#endif
 					return null;
 				}
 						
@@ -470,7 +480,9 @@ namespace System.Data.OleDb
 						i);
 					if (attrs == IntPtr.Zero){
 						// FIXME: throw exception
+#if DEBUG_OleDbDataReader
 						Console.WriteLine("Error: attrs null");
+#endif
 						return null;
 					}
 
