@@ -16,9 +16,10 @@ using System.Security.Permissions;
 
 namespace System.Data.OleDb
 {
-	[AttributeUsage(AttributeTargets.Assembly | AttributeTargets.Class
-			| AttributeTargets.Struct | AttributeTargets.Constructor |
-			AttributeTargets.Method)]
+	[AttributeUsage(AttributeTargets.Assembly | AttributeTargets.Class | 
+			AttributeTargets.Struct | AttributeTargets.Constructor |
+			AttributeTargets.Method, AllowMultiple=true,
+			Inherited=false)]
 	[Serializable]
 	public sealed class OleDbPermissionAttribute : DBDataPermissionAttribute
 	{
@@ -26,7 +27,7 @@ namespace System.Data.OleDb
 		#region Constructors 
 
 		[MonoTODO]
-		OleDbPermissionAttribute (SecurityAction action) 
+		public OleDbPermissionAttribute (SecurityAction action) 
 			: base (action)
 		{
 		}
