@@ -28,16 +28,18 @@ namespace System.Data.SqlClient {
 
 		#region Constructors
 
+		 [Obsolete ("Use SqlClientPermission(PermissionState.None)", true)]
 		public SqlClientPermission ()
-			: this (PermissionState.None, false)
+//			: this (PermissionState.None, false)
 		{
 		}
 
 		public SqlClientPermission (PermissionState state) 
-			: this (state, false)
+			: base (state)
 		{
 		}
 
+		[Obsolete ("Use SqlClientPermission(PermissionState.None)", true)]
 		public SqlClientPermission (PermissionState state, bool allowBlankPassword) 
 		{
 			AllowBlankPassword = allowBlankPassword;
@@ -49,7 +51,18 @@ namespace System.Data.SqlClient {
 
 		protected override DBDataPermission CreateInstance ()
 		{
-			return (DBDataPermission) new SqlClientPermission ();
+			return (DBDataPermission) new SqlClientPermission (PermissionState.None);
+		}
+
+		public override IPermission Copy()
+		{
+			return new SqlClientPermission ( state);			
+		}
+
+		[MonoTODO]
+		public override void Add (string connectionString, string restrictions, KeyRestrictionBehavior behavior)
+		{
+			throw new NotImplementedException ();
 		}
 
 		#endregion // Methods
