@@ -17,7 +17,10 @@ using System.Security;
 using System.Security.Permissions;
 
 namespace System.Data.SqlClient {
-	[AttributeUsage (AttributeTargets.Assembly | AttributeTargets.Class | AttributeTargets.Struct | AttributeTargets.Constructor | AttributeTargets.Method)]
+	[AttributeUsage (AttributeTargets.Assembly | AttributeTargets.Class | 
+			 AttributeTargets.Struct | AttributeTargets.Constructor | 
+			 AttributeTargets.Method, AllowMultiple=true,
+			 Inherited=false)]
 	[Serializable]
 	public sealed class SqlClientPermissionAttribute : DBDataPermissionAttribute 
 	{
@@ -32,10 +35,13 @@ namespace System.Data.SqlClient {
 
 		#region Methods
 
-		[MonoTODO]
 		public override IPermission CreatePermission() 
 		{
-			throw new NotImplementedException ();
+			if (base.Unrestricted) {
+				return new SqlClientPermission ( PermissionState.Unrestricted); 
+			}
+			return new SqlClientPermission ( PermissionState.None); 
+
 		}
 
 		#endregion // Methods
