@@ -36,10 +36,10 @@ namespace System.Data.SqlClient {
 		bool disposed = false;
 
 		// The set of SQL connection pools
-		static Hashtable SqlConnectionPools = new Hashtable ();
+		static TdsConnectionPoolManager sqlConnectionPools = new TdsConnectionPoolManager (TdsVersion.tds70);
 
 		// The current connection pool
-		SqlConnectionPool pool;
+		TdsConnectionPool pool;
 
 		// The connection string that identifies this connection
 		string connectionString = null;
@@ -88,6 +88,7 @@ namespace System.Data.SqlClient {
 		[DataCategory ("Data")]
 		[DataSysDescription ("Information used to connect to a DataSource, such as 'Data Source=x;Initial Catalog=x;Integrated Security=SSPI'.")]
 		[DefaultValue ("")]
+		[EditorAttribute ("Microsoft.VSDesigner.Data.SQL.Design.SqlConnectionStringEditor, "+ Consts.AssemblyMicrosoft_VSDesigner, "System.Drawing.Design.UITypeEditor, "+ Consts.AssemblySystem_Drawing )]
 		[RecommendedAsConfigurable (true)]	
 		[RefreshProperties (RefreshProperties.All)]
 		public string ConnectionString	{
@@ -210,11 +211,11 @@ namespace System.Data.SqlClient {
 			if (transaction != null)
 				throw new InvalidOperationException ("SqlConnection does not support parallel transactions.");
 
+			if (iso == IsolationLevel.Chaos)
+				throw new ArgumentException ("Invalid IsolationLevel parameter: must be ReadCommitted, ReadUncommitted, RepeatableRead, or Serializable.");
+
 			string isolevel = String.Empty;
 			switch (iso) {
-			case IsolationLevel.Chaos:
-				isolevel = "CHAOS";
-				break;
 			case IsolationLevel.ReadCommitted:
 				isolevel = "READ COMMITTED";
 				break;
@@ -268,8 +269,8 @@ namespace System.Data.SqlClient {
 				if(tds != null) tds.Disconnect ();
 
 			if(tds != null) {
-			tds.TdsErrorMessage -= new TdsInternalErrorMessageEventHandler (ErrorHandler);
-			tds.TdsInfoMessage -= new TdsInternalInfoMessageEventHandler (MessageHandler);
+				tds.TdsErrorMessage -= new TdsInternalErrorMessageEventHandler (ErrorHandler);
+				tds.TdsInfoMessage -= new TdsInternalInfoMessageEventHandler (MessageHandler);
 			}
 
 			ChangeState (ConnectionState.Closed);
@@ -351,14 +352,12 @@ namespace System.Data.SqlClient {
 					tds = new Tds70 (serverName, port, PacketSize, ConnectionTimeout);
 				}
 				else {
-					pool = (SqlConnectionPool) SqlConnectionPools [connectionString];
-					if (pool == null) {
-						if(!ParseDataSource (dataSource, out port, out serverName))
-							throw new SqlException(20, 0, "SQL Server does not exist or access denied.",  17, "ConnectionOpen (Connect()).", dataSource, parms.ApplicationName, 0);
-						pool = new SqlConnectionPool (serverName, port, packetSize, ConnectionTimeout, minPoolSize, maxPoolSize);
-						SqlConnectionPools [connectionString] = pool;
-					}
-					tds = pool.AllocateConnection ();
+					if(!ParseDataSource (dataSource, out port, out serverName))
+						throw new SqlException(20, 0, "SQL Server does not exist or access denied.",  17, "ConnectionOpen (Connect()).", dataSource, parms.ApplicationName, 0);
+					
+ 					TdsConnectionInfo info = new TdsConnectionInfo (serverName, port, packetSize, ConnectionTimeout, minPoolSize, maxPoolSize);
+					pool = sqlConnectionPools.GetConnectionPool (connectionString, info);
+					tds = pool.GetConnection ();
 				}
 			}
 			catch (TdsTimeoutException e) {
@@ -367,9 +366,18 @@ namespace System.Data.SqlClient {
 
 			tds.TdsErrorMessage += new TdsInternalErrorMessageEventHandler (ErrorHandler);
 			tds.TdsInfoMessage += new TdsInternalInfoMessageEventHandler (MessageHandler);
-
-			if (!tds.IsConnected) 
-				tds.Connect (parms);
+				
+			if (!tds.IsConnected) {
+				try {
+					tds.Connect (parms);
+				}
+				catch {
+					if (pooling)
+						pool.ReleaseConnection (tds);
+					throw;
+				}
+			}
+						
 			/* Not sure ebout removing these 2 lines.
 			 * The command that gets to the sql server is just
 			 * 'sp_reset_connection' and it fails.
@@ -423,54 +431,87 @@ namespace System.Data.SqlClient {
 
                 void SetConnectionString (string connectionString)
                 {
-                        connectionString += ";";
-                        NameValueCollection parameters = new NameValueCollection ();
+			NameValueCollection parameters = new NameValueCollection ();
 
-                        if (connectionString == String.Empty)
-                                return;
+			if (connectionString.Length == 0)
+				return;
+			connectionString += ";";
 
-                        bool inQuote = false;
-                        bool inDQuote = false;
+			bool inQuote = false;
+			bool inDQuote = false;
+			bool inName = true;
 
-                        string name = String.Empty;
-                        string value = String.Empty;
-                        StringBuilder sb = new StringBuilder ();
+			string name = String.Empty;
+			string value = String.Empty;
+			StringBuilder sb = new StringBuilder ();
 
-                        foreach (char c in connectionString)
-                        {
-                                switch (c) {
-                                case '\'':
-                                        inQuote = !inQuote;
-                                        break;
-                                case '"' :
-                                        inDQuote = !inDQuote;
-                                        break;
-                                case ';' :
-                                        if (!inDQuote && !inQuote) {
+			for (int i = 0; i < connectionString.Length; i += 1) {
+				char c = connectionString [i];
+				char peek;
+				if (i == connectionString.Length - 1)
+					peek = '\0';
+				else
+					peek = connectionString [i + 1];
+
+				switch (c) {
+				case '\'':
+					if (inDQuote)
+						sb.Append (c);
+					else if (peek.Equals (c)) {
+						sb.Append (c);
+						i += 1;
+					}
+					else
+						inQuote = !inQuote;
+					break;
+				case '"':
+					if (inQuote)
+						sb.Append (c);
+					else if (peek.Equals (c)) {
+						sb.Append (c);
+						i += 1;
+					}
+					else
+						inDQuote = !inDQuote;
+					break;
+				case ';':
+					if (inDQuote || inQuote)
+						sb.Append (c);
+					else {
 						if (name != String.Empty && name != null) {
-                                                	value = sb.ToString ();
-                                                	parameters [name.ToUpper ().Trim ()] = value.Trim ();
+							value = sb.ToString ();
+							parameters [name.ToUpper ().Trim ()] = value.Trim ();
 						}
-                                                name = String.Empty;
-                                                value = String.Empty;
-                                                sb = new StringBuilder ();
-                                        }
-                                        else
-                                                sb.Append (c);
-                                        break;
-                                case '=' :
-                                        if (!inDQuote && !inQuote) {
-                                                name = sb.ToString ();
-                                                sb = new StringBuilder ();
-                                        }
-                                        else
-                                                sb.Append (c);
-                                        break;
-                                default:
-                                        sb.Append (c);
-                                        break;
-                                }
-                        }
+						inName = true;
+						name = String.Empty;
+						value = String.Empty;
+						sb = new StringBuilder ();
+					}
+					break;
+				case '=':
+					if (inDQuote || inQuote || !inName)
+						sb.Append (c);
+					else if (peek.Equals (c)) {
+						sb.Append (c);
+						i += 1;
+					}
+					else {
+						name = sb.ToString ();
+						sb = new StringBuilder ();
+						inName = false;
+					}
+					break;
+				case ' ':
+					if (inQuote || inDQuote)
+						sb.Append (c);
+					else if (sb.Length > 0 && !peek.Equals (';'))
+						sb.Append (c);
+					break;
+				default:
+					sb.Append (c);
+					break;
+				}
+			}
 
                         if (this.ConnectionString == null)
                         {
@@ -515,8 +556,10 @@ namespace System.Data.SqlClient {
                 private void SetProperties (NameValueCollection parameters)
                 {
                         string value;
+			string upperValue;
                         foreach (string name in parameters) {
                                 value = parameters[name];
+				upperValue = value.ToUpper ();
 
                                 switch (name) {
                                 case "APPLICATION NAME" :
@@ -533,7 +576,7 @@ namespace System.Data.SqlClient {
                                 case "CONNECTION LIFETIME" :
                                         break;
                                 case "CONNECTION RESET" :
-                                        connectionReset = !(value.ToUpper ().Equals ("FALSE") || value.ToUpper ().Equals ("NO"));
+					connectionReset = !(upperValue.Equals ("FALSE") || upperValue.Equals ("NO"));
                                         break;
                                 case "CURRENT LANGUAGE" :
                                         parms.Language = value;
@@ -553,6 +596,7 @@ namespace System.Data.SqlClient {
                                         break;
                                 case "INTEGRATED SECURITY" :
                                 case "TRUSTED_CONNECTION" :
+					parms.DomainLogin = upperValue.Equals ("TRUE") || upperValue.Equals ("YES") || upperValue.Equals ("SSPI");
                                         break;
                                 case "MAX POOL SIZE" :
                                         maxPoolSize = Int32.Parse (value);
@@ -560,6 +604,10 @@ namespace System.Data.SqlClient {
                                 case "MIN POOL SIZE" :
                                         minPoolSize = Int32.Parse (value);
                                         break;
+#if NET_2_0
+				case "MULTIPLEACTIVERESULTSETS":
+					break;
+#endif
                                 case "NET" :
                                 case "NETWORK LIBRARY" :
                                         if (!value.ToUpper ().Equals ("DBMSSOCN"))
