@@ -34,6 +34,10 @@ namespace System.Data.SqlClient {
 		int resultsRead;
 		int rowsRead;
 		DataTable schemaTable;
+		bool hasRows;
+		bool haveRead;
+		bool readResult;
+		bool readResultUsed;
 
 		#endregion // Fields
 
@@ -41,6 +45,9 @@ namespace System.Data.SqlClient {
 
 		internal SqlDataReader (SqlCommand command)
 		{
+			readResult = false;
+			haveRead = false;
+			readResultUsed = false;
 			this.command = command;
 			schemaTable = ConstructSchemaTable ();
 			resultsRead = 0;
@@ -84,6 +91,17 @@ namespace System.Data.SqlClient {
 			}
 		}
 
+		public bool HasRows {
+			get {
+				if (haveRead) 
+					return readResult;
+			
+				haveRead = true;
+				readResult = ReadRecord ();
+				return readResult;						
+			}
+		}
+	
 		#endregion // Properties
 
 		#region Methods
@@ -146,24 +164,30 @@ namespace System.Data.SqlClient {
 		public bool GetBoolean (int i)
 		{
 			object value = GetValue (i);
-			if (!(value is bool))
+			if (!(value is bool)) {
+				if (value is DBNull) throw new SqlNullValueException ();
 				throw new InvalidCastException ("Type is " + value.GetType ().ToString ());
+			}
 			return (bool) value;
 		}
 
 		public byte GetByte (int i)
 		{
 			object value = GetValue (i);
-			if (!(value is byte))
+			if (!(value is byte)) {
+				if (value is DBNull) throw new SqlNullValueException ();
 				throw new InvalidCastException ("Type is " + value.GetType ().ToString ());
+			}
 			return (byte) value;
 		}
 
 		public long GetBytes (int i, long dataIndex, byte[] buffer, int bufferIndex, int length)
 		{
 			object value = GetValue (i);
-			if (!(value is byte []))
+			if (!(value is byte [])) {
+				if (value is DBNull) throw new SqlNullValueException ();
 				throw new InvalidCastException ("Type is " + value.GetType ().ToString ());
+			}
 			
 			if ( buffer == null ) {
 				// Return length of data
@@ -175,12 +199,15 @@ namespace System.Data.SqlClient {
 				return ((byte []) value).Length - dataIndex;
 			}
 		}
-
+                                                                                                    
+		[EditorBrowsableAttribute (EditorBrowsableState.Never)]
 		public char GetChar (int i)
 		{
 			object value = GetValue (i);
-			if (!(value is char))
+			if (!(value is char)) {
+				if (value is DBNull) throw new SqlNullValueException ();
 				throw new InvalidCastException ("Type is " + value.GetType ().ToString ());
+			}
 			return (char) value;
 		}
 
@@ -193,8 +220,10 @@ namespace System.Data.SqlClient {
 				valueBuffer = (char[])value;
 			else if (value is string)
 				valueBuffer = ((string)value).ToCharArray();
-			else
+			else {
+				if (value is DBNull) throw new SqlNullValueException ();
 				throw new InvalidCastException ("Type is " + value.GetType ().ToString ());
+			}
 			
 			if ( buffer == null ) {
 				// Return length of data
@@ -206,11 +235,11 @@ namespace System.Data.SqlClient {
 				return valueBuffer.Length - dataIndex;
 			}
 		}
-
-		[MonoTODO ("Implement GetData")]
+		
+		[EditorBrowsableAttribute (EditorBrowsableState.Never)] 
 		public IDataReader GetData (int i)
 		{
-			throw new NotImplementedException ();
+			return ( (IDataReader) this [i]);
 		}
 
 		public string GetDataTypeName (int i)
@@ -221,24 +250,30 @@ namespace System.Data.SqlClient {
 		public DateTime GetDateTime (int i)
 		{
 			object value = GetValue (i);
-			if (!(value is DateTime))
-				throw new InvalidCastException ("Type is " + value.GetType ().ToString ());
+			if (!(value is DateTime)) {
+				if (value is DBNull) throw new SqlNullValueException ();
+				else throw new InvalidCastException ("Type is " + value.GetType ().ToString ());
+			}
 			return (DateTime) value;
 		}
 
 		public decimal GetDecimal (int i)
 		{
 			object value = GetValue (i);
-			if (!(value is decimal))
+			if (!(value is decimal)) {
+				if (value is DBNull) throw new SqlNullValueException ();
 				throw new InvalidCastException ("Type is " + value.GetType ().ToString ());
+			}
 			return (decimal) value;
 		}
 
 		public double GetDouble (int i)
 		{
 			object value = GetValue (i);
-			if (!(value is double))
+			if (!(value is double)) {
+				if (value is DBNull) throw new SqlNullValueException ();
 				throw new InvalidCastException ("Type is " + value.GetType ().ToString ());
+			}
 			return (double) value;
 		}
 
@@ -250,40 +285,50 @@ namespace System.Data.SqlClient {
 		public float GetFloat (int i)
 		{
 			object value = GetValue (i);
-			if (!(value is float))
+			if (!(value is float)) {
+				if (value is DBNull) throw new SqlNullValueException ();
 				throw new InvalidCastException ("Type is " + value.GetType ().ToString ());
+			}
 			return (float) value;
 		}
 
 		public Guid GetGuid (int i)
 		{
 			object value = GetValue (i);
-			if (!(value is Guid))
+			if (!(value is Guid)) {
+				if (value is DBNull) throw new SqlNullValueException ();
 				throw new InvalidCastException ("Type is " + value.GetType ().ToString ());
+			}
 			return (Guid) value;
 		}
 
 		public short GetInt16 (int i)
 		{
 			object value = GetValue (i);
-			if (!(value is short))
+			if (!(value is short)) {
+				if (value is DBNull) throw new SqlNullValueException ();
 				throw new InvalidCastException ("Type is " + value.GetType ().ToString ());
+			}
 			return (short) value;
 		}
 
 		public int GetInt32 (int i)
 		{
 			object value = GetValue (i);
-			if (!(value is int))
+			if (!(value is int)) {
+				if (value is DBNull) throw new SqlNullValueException ();
 				throw new InvalidCastException ("Type is " + value.GetType ().ToString ());
+			}
 			return (int) value;
 		}
 
 		public long GetInt64 (int i)
 		{
 			object value = GetValue (i);
-			if (!(value is long))
+			if (!(value is long)) {
+				if (value is DBNull) throw new SqlNullValueException ();
 				throw new InvalidCastException ("Type is " + value.GetType ().ToString ());
+			}
 			return (long) value;
 		}
 
@@ -616,7 +661,6 @@ namespace System.Data.SqlClient {
 			return (SqlString) value;
 		}
 
-		[MonoTODO ("Implement TdsBigDecimal conversion.  SqlDbType.Real fails tests?")]
 		public object GetSqlValue (int i)
 		{
 			SqlDbType type = (SqlDbType) (schemaTable.Rows [i]["ProviderType"]);
@@ -624,18 +668,18 @@ namespace System.Data.SqlClient {
 
 			switch (type) {
 			case SqlDbType.BigInt:
-				if (value == null)
+				if (value == DBNull.Value)
 					return SqlInt64.Null;
 				return (SqlInt64) ((long) value);
 			case SqlDbType.Binary:
 			case SqlDbType.Image:
 			case SqlDbType.VarBinary:
 			case SqlDbType.Timestamp:
-				if (value == null)
+				if (value == DBNull.Value)
 					return SqlBinary.Null;
 				return (SqlBinary) ((byte[]) value);
 			case SqlDbType.Bit:
-				if (value == null)
+				if (value == DBNull.Value)
 					return SqlBoolean.Null;
 				return (SqlBoolean) ((bool) value);
 			case SqlDbType.Char:
@@ -644,47 +688,47 @@ namespace System.Data.SqlClient {
 			case SqlDbType.NVarChar:
 			case SqlDbType.Text:
 			case SqlDbType.VarChar:
-				if (value == null)
+				if (value == DBNull.Value)
 					return SqlString.Null;
 				return (SqlString) ((string) value);
 			case SqlDbType.DateTime:
 			case SqlDbType.SmallDateTime:
-				if (value == null)
+				if (value == DBNull.Value)
 					return SqlDateTime.Null;
 				return (SqlDateTime) ((DateTime) value);
 			case SqlDbType.Decimal:
-				if (value == null)
+				if (value == DBNull.Value)
 					return SqlDecimal.Null;
 				if (value is TdsBigDecimal)
 					return SqlDecimal.FromTdsBigDecimal ((TdsBigDecimal) value);
 				return (SqlDecimal) ((decimal) value);
 			case SqlDbType.Float:
-				if (value == null)
+				if (value == DBNull.Value)
 					return SqlDouble.Null;
 				return (SqlDouble) ((double) value);
 			case SqlDbType.Int:
-				if (value == null)
+				if (value == DBNull.Value)
 					return SqlInt32.Null;
 				return (SqlInt32) ((int) value);
 			case SqlDbType.Money:
 			case SqlDbType.SmallMoney:
-				if (value == null)
+				if (value == DBNull.Value)
 					return SqlMoney.Null;
 				return (SqlMoney) ((decimal) value);
 			case SqlDbType.Real:
-				if (value == null)
+				if (value == DBNull.Value)
 					return SqlSingle.Null;
 				return (SqlSingle) ((float) value);
 			case SqlDbType.UniqueIdentifier:
-				if (value == null)
+				if (value == DBNull.Value)
 					return SqlGuid.Null;
 				return (SqlGuid) ((Guid) value);
 			case SqlDbType.SmallInt:
-				if (value == null)
+				if (value == DBNull.Value)
 					return SqlInt16.Null;
 				return (SqlInt16) ((short) value);
 			case SqlDbType.TinyInt:
-				if (value == null)
+				if (value == DBNull.Value)
 					return SqlByte.Null;
 				return (SqlByte) ((byte) value);
 			}
@@ -712,8 +756,10 @@ namespace System.Data.SqlClient {
 		public string GetString (int i)
 		{
 			object value = GetValue (i);
-			if (!(value is string))
+			if (!(value is string)) {
+				if (value is DBNull) throw new SqlNullValueException ();
 				throw new InvalidCastException ("Type is " + value.GetType ().ToString ());
+			}
 			return (string) value;
 		}
 
@@ -749,7 +795,7 @@ namespace System.Data.SqlClient {
 
 		public bool IsDBNull (int i)
 		{
-			return GetValue (i) == null;
+			return GetValue (i) == DBNull.Value;
 		}
 
 		public bool NextResult ()
@@ -778,13 +824,29 @@ namespace System.Data.SqlClient {
 				return false;
 			if (!moreResults)
 				return false;
+	
+			if ((haveRead) && (!readResultUsed))
+			{
+				readResultUsed = true;
+				return true;
+			}
+			
+			
+			return (ReadRecord ());
 
+		}
+
+		internal bool ReadRecord ()
+		{
+			
 			bool result = command.Tds.NextRow ();
-
+			
 			rowsRead += 1;
-
+			
 			return result;
 		}
+				
+		
 
 		#endregion // Methods
 	}
