@@ -20,6 +20,7 @@ using System.Collections;
 
 namespace System.Data.SqlClient {
 	[ListBindable (false)]
+	[EditorAttribute ("Microsoft.VSDesigner.Data.Design.DataParametersEditor, "+ Consts.AssemblyMicrosoft_VSDesigner, "System.Drawing.Design.UITypeEditor, "+ Consts.AssemblySystem_Drawing )]
 	public sealed class SqlParameterCollection : MarshalByRefObject, IDataParameterCollection, IList, ICollection, IEnumerable
 	{
 		#region Fields
@@ -192,13 +193,11 @@ namespace System.Data.SqlClient {
 		
 		public int IndexOf (string parameterName)
 		{
-			int i=0;
-			for( ; i < list.Count; i++ ) {
-				if (((SqlParameter )list[i]).ParameterName.Equals (parameterName))
-					break;
-			}
+			for (int i = 0; i < Count; i += 1)
+                                if (this [i].ParameterName.Equals (parameterName))
+                                        return i;
+                        return -1;
 
-			return i;
 		}
 
 		public void Insert (int index, object value)
