@@ -90,13 +90,25 @@ namespace System.Data.SqlTypes
 			precision = GetPrecision (value);
 		}
 				
-		public SqlDecimal (double value) : this ((decimal)value) { }
+		public SqlDecimal (double value) : this ((decimal)value) 
+		{
+			SqlDecimal n = this;
+			int digits = 17 - precision;
+			if (digits > 0)
+				n = AdjustScale (this, digits, false);
+			else
+				n = Round (this, 17);
+			this.notNull = n.notNull;
+			this.positive = n.positive;
+			this.precision = n.precision;
+			this.scale = n.scale;
+			this.value = n.value;
+		}
 		public SqlDecimal (int value) : this ((decimal)value) { }
 		public SqlDecimal (long value) : this ((decimal)value) { }
 
 		public SqlDecimal (byte bPrecision, byte bScale, bool fPositive, int[] bits) : this (bPrecision, bScale, fPositive, bits[0], bits[1], bits[2], bits[3]) { }
 
-		[MonoTODO]
 		public SqlDecimal (byte bPrecision, byte bScale, bool fPositive, int data1, int data2, int data3, int data4) 
 		{
 			this.precision = bPrecision;
@@ -108,14 +120,16 @@ namespace System.Data.SqlTypes
 			this.value[2] = data3;
 			this.value[3] = data4;
 			notNull = true;
-			
-			if (precision < scale)
-				throw new ArgumentException(Locale.GetText ("Invalid scale"));
 
-			// FIXME: What is the right message of Exception
+			if (precision < scale)
+				throw new SqlTypeException (Locale.GetText ("Invalid presicion/scale combination."));
+
+			if (precision > 38)
+				throw new SqlTypeException (Locale.GetText ("Invalid precision/scale combination."));
+
 			if (this.ToDouble () > (Math.Pow (10, 38) - 1)  || 
 			    this.ToDouble () < -(Math.Pow (10, 38)))
-				throw new SqlTypeException ("Can't convert to SqlDecimal");
+				throw new SqlTypeException ("Can't convert to SqlDecimal, Out of range ");
 		}
 
 		#endregion
@@ -144,8 +158,13 @@ namespace System.Data.SqlTypes
 			get { 
 				if (this.IsNull)
 					throw new SqlNullValueException ();
-				else
-					return (value);
+				// Data should always return clone, not to be modified
+				int [] ret = new int [4];
+				ret [0] = value [0];
+				ret [1] = value [1];
+				ret [2] = value [2];
+				ret [3] = value [3];
+				return ret;
 			}
 		}
 
@@ -183,9 +202,9 @@ namespace System.Data.SqlTypes
 
 		public static SqlDecimal Abs (SqlDecimal n)
 		{
-				return new SqlDecimal (n.Precision, n.Scale, true, 
-						       n.BinData [0], n.BinData [1], 
-						       n.BinData [2], n.BinData [3]);
+			if (!n.notNull)
+				return n;
+			return new SqlDecimal (n.Precision, n.Scale, true, n.Data);
 		}
 
 		public static SqlDecimal Add (SqlDecimal x, SqlDecimal y)
@@ -199,19 +218,35 @@ namespace System.Data.SqlTypes
 			if (n.IsNull)
 				throw new SqlNullValueException ();
 
-			if (digits > 0)
+			int [] data;
+			byte newScale;
+			if (digits == 0)
+				return n;
+			else if (digits > 0) {
 			        prec = (byte)(prec + digits);
+				decimal d = n.Value;
+				if (digits > 0)
+					for (int i = 0; i < digits; i++)
+						d *= 10;
+				data = Decimal.GetBits (d);
+				data [3] = 0;
+				newScale = (byte) (n.scale + digits);
+			} else {
+				if (fRound)
+					n = Round (n, digits + n.scale);
+				else
+					n = Round (Truncate (n, digits + n.scale), digits + n.scale);
+				data = n.Data;
+				newScale = n.scale;
+			}
 
-			if (fRound)
-				n = Round (n, digits + n.Scale);
-
-			return new SqlDecimal (prec, 
-					       (byte)(n.Scale + digits), 
-					       n.IsPositive, n.Data);
+			return new SqlDecimal (prec, newScale, n.positive, data);
 		}
 
 		public static SqlDecimal Ceiling (SqlDecimal n)
 		{
+			if (!n.notNull)
+				return n;
 			return AdjustScale (n, -(n.Scale), true);
 		}
 
@@ -229,7 +264,9 @@ namespace System.Data.SqlTypes
 
 		public static SqlDecimal ConvertToPrecScale (SqlDecimal n, int precision, int scale)
 		{
-			return new SqlDecimal ((byte)precision, (byte)scale, n.IsPositive, n.Data);
+//			return new SqlDecimal ((byte)precision, (byte)scale, n.IsPositive, n.Data);
+			// FIXME: precision
+			return AdjustScale (n, scale - n.scale, true);
 		}
 
 		public static SqlDecimal Divide (SqlDecimal x, SqlDecimal y)
@@ -310,14 +347,12 @@ namespace System.Data.SqlTypes
 			return (x != y);
 		}
 
-		[MonoTODO]
 		public static SqlDecimal Parse (string s)
 		{
-			// FIXME: Huh, There must be better way to do this
 			if (s == null)
-				throw new ArgumentNullException ();
+				throw new ArgumentNullException (Locale.GetText ("string s"));
 			else 
-				return SqlDouble.Parse (s).ToSqlDecimal ();
+				return new SqlDecimal (Decimal.Parse (s));
 		}
 
 		public static SqlDecimal Power (SqlDecimal n, double exp)
@@ -328,19 +363,14 @@ namespace System.Data.SqlTypes
 			return new SqlDecimal (Math.Pow (n.ToDouble (), exp));
 		}
 
-		[MonoTODO]
 		public static SqlDecimal Round (SqlDecimal n, int position)
 		{
-			// FIXME: There must be better way to do this
 			if (n.IsNull)
 				throw new SqlNullValueException ();
 
-			SqlDecimal result = new SqlDecimal (Math.Round (
-				(double)(n.ToDouble () * Math.Pow (10, position))));
-
-			result = result / new SqlDecimal(Math.Pow (10, position));
-			
-			return result;				
+			decimal d = n.Value;
+			d = Decimal.Round (d, position);
+			return new SqlDecimal (d);
 		}
 
 		public static SqlInt32 Sign (SqlDecimal n)
@@ -434,7 +464,7 @@ namespace System.Data.SqlTypes
 		public override string ToString ()
 		{
 			if (this.IsNull)
-				return String.Empty;
+				return "Null";
 			
 			// convert int [4] --> ulong [2]
 			ulong lo = (uint)this.Data [0];
@@ -460,6 +490,9 @@ namespace System.Data.SqlTypes
 			if (this.Scale > 0)
 			        Result.Insert (Result.Length - this.Scale, ".");
 
+			if (!positive)
+				Result.Insert (0, '-');
+
 			return Result.ToString ();
 		}
 
@@ -511,6 +544,7 @@ namespace System.Data.SqlTypes
 			int texp = 0;
 			int rc = 0;
 			byte prec = 0; // precision
+			bool positive = ! (x.positive ^ y.positive);
 
 			prec = x.Precision >= y.Precision ? x.Precision : y.Precision;
 			DecimalDivSub (ref x, ref y, ref lo, ref hi, ref texp);
@@ -543,7 +577,7 @@ namespace System.Data.SqlTypes
 			int resultMi2 = (int)(hi);
 			int resultHi = (int)(hi >> 32);
 
-			return new SqlDecimal (prec, (byte)sc, true, resultLo,
+			return new SqlDecimal (prec, (byte)sc, positive, resultLo,
 						       resultMi, resultMi2,
 						       resultHi);
 		}
@@ -593,9 +627,9 @@ namespace System.Data.SqlTypes
 
 					// 10^i/2^i=5^i 
 					factor = constantsDecadeInt32Factors [i] >> i; 
-					System.Console.WriteLine ("***");
+//					System.Console.WriteLine ("***");
 					Mult128By32 (ref clo, ref chi, factor, 0);
-			       		System.Console.WriteLine ((((double)chi) * Math.Pow (2,64) + clo));
+//			       		System.Console.WriteLine ((((double)chi) * Math.Pow (2,64) + clo));
 
 				}
 
@@ -992,10 +1026,17 @@ namespace System.Data.SqlTypes
 
 		public static SqlDecimal Truncate (SqlDecimal n, int position)
 		{
-			int prec = n.Precision;// + (position - n.Scale);
-			int sc = position;
-			return new SqlDecimal ((byte)prec, (byte)sc,
-					       n.IsPositive, n.Data);
+			int diff = n.scale - position;
+			if (diff == 0)
+				return n;
+			int [] data = n.Data;
+			decimal d = new decimal (data [0], data [1], data [2], !n.positive, 0);
+			decimal x = 10;
+			for (int i = 0; i < diff; i++, x *= 10)
+				d = d - d % x;
+			data = Decimal.GetBits (d);
+			data [3] = 0;
+			return new SqlDecimal (n.precision, n.scale, n.positive, data);
 		}
 
 		public static SqlDecimal operator + (SqlDecimal x, SqlDecimal y)
@@ -1003,12 +1044,12 @@ namespace System.Data.SqlTypes
 			// if one of them is negative, perform subtraction
 			if (x.IsPositive && !y.IsPositive) return x - y;
 			if (y.IsPositive && !x.IsPositive) return y - x;
-		
-			// adjust the scale to the smaller of the two beforehand
-			if (x.Scale > y.Scale)
-				x = SqlDecimal.AdjustScale(x, y.Scale - x.Scale, true);
-			else if (y.Scale > x.Scale)
-				y = SqlDecimal.AdjustScale(y, x.Scale - y.Scale, true);
+
+			// adjust the scale to the larger of the two beforehand
+			if (x.scale > y.scale)
+				y = SqlDecimal.AdjustScale (y, x.scale - y.scale, true); // FIXME: should be false (fix it after AdjustScale(,,false) is fixed)
+			else if (y.scale > x.scale)
+				x = SqlDecimal.AdjustScale (x, y.scale - x.scale, true); // FIXME: should be false (fix it after AdjustScale(,,false) is fixed)
 
 			// set the precision to the greater of the two
 			byte resultPrecision;
@@ -1255,7 +1296,7 @@ namespace System.Data.SqlTypes
 				if (x.IsNull) 
 					return Null;
 				else
-					return new SqlDecimal ((decimal)x.Value);
+					return new SqlDecimal ((double)x.Value);
 			}
 		}
 
@@ -1265,7 +1306,7 @@ namespace System.Data.SqlTypes
 				if (x.IsNull) 
 					return Null;
 				else
-					return new SqlDecimal ((decimal)x.Value);
+					return new SqlDecimal ((double)x.Value);
 			}
 		}
 
