@@ -12,10 +12,10 @@ using System.Runtime.Serialization;
 namespace System.Data.SqlTypes {
 
 	[Serializable]
-	public class SqlNullValueException : SqlTypeException
+	public sealed class SqlNullValueException : SqlTypeException
 	{
 		public SqlNullValueException ()
-			: base (Locale.GetText ("The value property is null"))
+			: base (Locale.GetText ("Data is Null. This method or property cannot be called on Null values."))
 		{
 		}
 
@@ -23,5 +23,10 @@ namespace System.Data.SqlTypes {
 			: base (message)
 		{
 		}
+
+		public SqlNullValueException (string message, Exception inner)
+			: base (message, inner)
+		{
+		}
 	}
 }
