@@ -14,6 +14,12 @@ namespace System.Data.SqlTypes {
 	[Serializable]			
 	public class SqlTypeException : SystemException
 	{
+		
+		public SqlTypeException()
+			: base (Locale.GetText ("A sql exception has occured."))
+		{
+		}
+
 		public SqlTypeException (string message)
 			: base (message)
 		{
@@ -23,5 +29,11 @@ namespace System.Data.SqlTypes {
 			: base (info, context)
 		{
 		}
+
+		public SqlTypeException (string message, Exception inner)
+                        : base (message, inner)
+                {
+                }
+
 	}
 }
