@@ -23,6 +23,7 @@ namespace System.Data {
 	/// hold collection of constraints for data table
 	/// </summary>
 	[DefaultEvent ("CollectionChanged")]
+	[EditorAttribute("Microsoft.VSDesigner.Data.Design.ConstraintsCollectionEditor, "+Consts.AssemblyMicrosoft_VSDesigner, "System.Drawing.Design.UITypeEditor, "+Consts.AssemblySystem_Drawing )]
 	[Serializable]
 	public class ConstraintCollection : InternalDataCollectionBase 
 	{
@@ -31,6 +32,16 @@ namespace System.Data {
 		public event CollectionChangeEventHandler CollectionChanged;
 		internal event DelegateValidateRemoveConstraint ValidateRemoveConstraint;
 		private DataTable table;
+		
+		// Call this to set the "table" property of the UniqueConstraint class
+                // intialized with UniqueConstraint( string, string[], bool );
+                // And also validate that the named columns exist in the "table"
+                private delegate void PostAddRange( DataTable table );
+		
+		// Keep reference to most recent constraints passed to AddRange()
+                // so that they can be added when EndInit() is called.
+                private Constraint [] _mostRecentConstraints;
+
 		//Don't allow public instantiation
 		//Will be instantianted from DataTable
 		internal ConstraintCollection(DataTable table){
@@ -72,16 +83,10 @@ namespace System.Data {
 		}
 
 		private bool _isDuplicateConstraintName(string constraintName, Constraint excludeFromComparison) 
-		{	
-			string cmpr = constraintName.ToUpper();
-			foreach (Constraint cst in List) 
-			{
-				//Case insensitive comparision
-				if (  cmpr.CompareTo(cst.ConstraintName.ToUpper()) == 0  &&
-					cst != excludeFromComparison) 
-				{
+		{
+			foreach (Constraint cst in List) {
+				if (String.Compare (constraintName, cst.ConstraintName, !Table.CaseSensitive) == 0  && cst != excludeFromComparison) 
 					return true;
-				}
 			}
 
 			return false;
@@ -100,11 +105,15 @@ namespace System.Data {
 				foreach (Constraint cst in List) 
 				{
 					//Case insensitive
-					if (cst.ConstraintName.ToUpper().CompareTo("CONSTRAINT" + 
-						index.ToString()) == 0 ) 
+					if (String.Compare (cst.ConstraintName,
+						"Constraint" + index,
+						!Table.CaseSensitive,
+						Table.Locale)
+						== 0)
 					{
 						loopAgain = true;
 						index++;
+						break;
 					}
 				}
 			} while (loopAgain);
@@ -130,14 +139,39 @@ namespace System.Data {
 				throw new ArgumentException("Constraint already belongs to another collection.");
 			
 			//check for duplicate name
+#if !NET_1_1
 			if (_isDuplicateConstraintName(constraint.ConstraintName,null)  )
 				throw new DuplicateNameException("Constraint name already exists.");
-		
+#endif
+	
+			// Check whether Constraint is UniqueConstraint and initailized with the special
+                        // constructor - UniqueConstraint( string, string[], bool );
+                        // If yes, It must be added via AddRange() only
+                        // Environment.StackTrace can help us 
+			// FIXME: Is a different mechanism to do this?
+                        if (constraint is UniqueConstraint){
+                                if ((constraint as UniqueConstraint).DataColsNotValidated == true){
+                                        if ( Environment.StackTrace.IndexOf( "AddRange" ) == -1 ){
+                                                throw new ArgumentException(" Some DataColumns are invalid - They may not belong to the table associated with this Constraint Collection" );
+                                        }
+                                }
+                        }
+
+			  if (constraint is ForeignKeyConstraint){
+                                if ((constraint as ForeignKeyConstraint).DataColsNotValidated == true){
+                                        if ( Environment.StackTrace.IndexOf( "AddRange" ) == -1 ){
+                                                throw new ArgumentException(" Some DataColumns are invalid - They may not belong to the table associated with this Constraint Collection" );
+                                        }
+                                }
+                        }
+ 	
 			//Allow constraint to run validation rules and setup 
 			constraint.AddToConstraintCollectionSetup(this); //may throw if it can't setup
 
 			//Run Constraint to check existing data in table
-			constraint.AssertConstraint();
+			// this is redundant, since AddToConstraintCollectionSetup 
+			// calls AssertConstraint right before this call
+			//constraint.AssertConstraint();
 
 			//if name is null or empty give it a name
 			if (constraint.ConstraintName == null || 
@@ -198,12 +232,57 @@ namespace System.Data {
 			return fc;
 		}
 
-		[MonoTODO]
 		public void AddRange(Constraint[] constraints) {
 
-			throw new NotImplementedException ();
+			//When AddRange() occurs after BeginInit,
+                        //it does not add any elements to the collection until EndInit is called.
+                        if (this.table.InitStatus == DataTable.initStatus.BeginInit){
+                                // Keep reference so that they can be added when EndInit() is called.
+                                _mostRecentConstraints = constraints;
+                                return;
+                        }
+                        
+                        // Check whether the constraint is UniqueConstraint
+                        // And whether it was initialized with the special ctor
+                        // i.e UniqueConstraint( string, string[], bool );
+                        for (int i = 0; i < constraints.Length; i++){
+                                if (constraints[i] is UniqueConstraint){
+                                        if (( constraints[i] as UniqueConstraint).DataColsNotValidated == true){
+                                                PostAddRange _postAddRange= new PostAddRange ((constraints[i] as UniqueConstraint).PostAddRange);
+                                                // UniqueConstraint.PostAddRange() validates whether all named
+                                                // columns exist in the table associated with this instance of
+                                                // ConstraintCollection.
+                                                _postAddRange (this.table);
+                                                                                                    
+                                        }
+                                }
+				else if (constraints [i] is ForeignKeyConstraint){
+                                        if (( constraints [i] as ForeignKeyConstraint).DataColsNotValidated == true){
+                                                (constraints [i] as ForeignKeyConstraint).postAddRange (this.table);
+                                        }
+                                }
+	
+                        }
+                                                                                                    
+                        if ( (constraints == null) || (constraints.Length == 0))
+                                throw new ArgumentNullException ("Cannot add null");
+
+			 else {
+                                foreach (Constraint constraint in constraints)
+                                         Add (constraint);
+                                       
+                               }
+
+				
 		}
 
+		// Helper AddRange() - Call this function when EndInit is called
+                internal void PostEndInit()
+                {
+                        AddRange (_mostRecentConstraints);
+                }
+
+
 		public bool CanRemove(Constraint constraint) 
 		{
 
@@ -226,7 +305,6 @@ namespace System.Data {
 			
 		}
 
-		[MonoTODO]
 		public void Clear() 
 		{
 			
@@ -245,6 +323,7 @@ namespace System.Data {
 			//LAMESPEC: MSFT implementation allows this
 			//even when a ForeignKeyConstraint exist for a UniqueConstraint
 			//thus violating the CanRemove logic
+			//CanRemove will throws Exception incase of the above
 			List.Clear(); //Will violate CanRemove rule
 			OnCollectionChanged( new CollectionChangeEventArgs(CollectionChangeAction.Refresh, this) );
 		}
@@ -268,7 +347,7 @@ namespace System.Data {
 			int index = 0;
 			foreach (Constraint con in List)
 			{
-				if (constraintName.ToUpper().CompareTo( con.ConstraintName.ToUpper() ) == 0)
+				if (String.Compare (constraintName, con.ConstraintName, !Table.CaseSensitive, Table.Locale) == 0)
 				{
 					return index;
 				}
@@ -345,5 +424,32 @@ namespace System.Data {
 			return !cancel;
 		}
 
+		internal ICollection UniqueConstraints
+		{
+			get
+			{ 
+				return GetConstraintsCollection(typeof(UniqueConstraint));
+			}
+		}
+
+		internal ICollection ForeignKeyConstraints
+		{
+			get
+			{ 
+				return GetConstraintsCollection(typeof(ForeignKeyConstraint));
+			}
+		}
+
+		private ICollection GetConstraintsCollection (Type constraintType)
+		{
+			ArrayList cCollection = new ArrayList();
+			foreach (Constraint c in List) 
+			{
+				if (c.GetType() == constraintType)
+					cCollection.Add(c);
+			}
+			return cCollection;
+		}
+
 	}
 }
