@@ -18,6 +18,9 @@
 using System;
 using System.ComponentModel;
 using System.Reflection;
+using System.Collections;
+using System.Data.Common;
+using Mono.Data.SqlExpressions;
 
 namespace System.Data {
 	internal delegate void DelegateColumnValueChange(DataColumn column, DataRow row, object proposedValue);
@@ -28,7 +31,6 @@ namespace System.Data {
 
 	[Editor]
 	[ToolboxItem (false)]
-	[DefaultMember ("Item")]
 	[DefaultProperty ("ColumnName")]
 	[DesignTimeVisible (false)]
 	public class DataColumn : MarshalByValueComponent
@@ -46,65 +48,87 @@ namespace System.Data {
 		#region Fields
 
 		private bool _allowDBNull = true;
-		private bool _autoIncrement = false;
-		private long _autoIncrementSeed = 0;
+		private bool _autoIncrement;
+		private long _autoIncrementSeed;
 		private long _autoIncrementStep = 1;
-		private long _nextAutoIncrementValue = 0;
-		private bool dataHasBeenSet = false;
-		private string _caption = null;
-		private MappingType _columnMapping = MappingType.Element;
-		private string _columnName = null;
-		private Type _dataType = Type.GetType ("System.String");
+		private long _nextAutoIncrementValue;
+		private string _caption;
+		private MappingType _columnMapping;
+		private string _columnName;
 		private object _defaultValue = DBNull.Value;
-		private string expression = "";
+		private string expression;
+		private IExpression compiledExpression;
 		private PropertyCollection _extendedProperties = new PropertyCollection ();
 		private int maxLength = -1; //-1 represents no length limit
-		private string nameSpace = "";
+		private string nameSpace = String.Empty;
 		private int _ordinal = -1; //-1 represents not part of a collection
-		private string prefix = "";
-		private bool readOnly = false;
-		private DataTable _table = null;
-		private bool unique = false;
+		private string prefix = String.Empty;
+		private bool readOnly;
+		private DataTable _table;
+		private bool unique;
+		private AbstractDataContainer _dataContainer;
 
 		#endregion // Fields
 
 		#region Constructors
 
-		public DataColumn()
+		public DataColumn() : this(String.Empty, typeof (string), String.Empty, MappingType.Element)
 		{
 		}
 
 		//TODO: Ctor init vars directly
-		public DataColumn(string columnName): this()
+		public DataColumn(string columnName): this(columnName, typeof (string), String.Empty, MappingType.Element)
 		{
-			ColumnName = columnName;
 		}
 
-		public DataColumn(string columnName, Type dataType): this(columnName)
+		public DataColumn(string columnName, Type dataType): this(columnName, dataType, String.Empty, MappingType.Element)
 		{
+		}
+
+		public DataColumn( string columnName, Type dataType, 
+			string expr): this(columnName, dataType, expr, MappingType.Element)
+		{
+		}
+
+		public DataColumn(string columnName, Type dataType, 
+			string expr, MappingType type)
+		{
+			ColumnName = (columnName == null ? String.Empty : columnName);
+			
 			if(dataType == null) {
 				throw new ArgumentNullException("dataType can't be null.");
 			}
 			
 			DataType = dataType;
-
-		}
-
-		public DataColumn( string columnName, Type dataType, 
-			string expr): this(columnName, dataType)
-		{
-			if (expr != null) Expression = expr;
-		}
-
-		public DataColumn(string columnName, Type dataType, 
-			string expr, MappingType type): this(columnName, dataType, expr)
-		{
+			Expression = expr == null ? String.Empty : expr;
 			ColumnMapping = type;
 		}
 		#endregion
 
 		#region Properties
 
+		internal object this[int index] {
+			get {
+				return DataContainer[index];
+			}
+			set {
+				DataContainer[index] = value;
+
+				if ( AutoIncrement && !DataContainer.IsNull(index) ) {
+					long value64 = Convert.ToInt64(value);
+					if (_autoIncrementStep > 0 ) {
+						if (value64 >= _nextAutoIncrementValue) {
+							_nextAutoIncrementValue = value64;
+							AutoIncrementValue ();
+						}
+					}
+					else if (value64 <= _nextAutoIncrementValue) {
+						AutoIncrementValue ();
+					}
+				}
+			}
+		}
+
 		[DataCategory ("Data")]
 		[DataSysDescription ("Indicates whether null values are allowed in this column.")]
 		[DefaultValue (true)]
@@ -177,13 +201,21 @@ namespace System.Data {
 
 					//If the DataType of this Column isn't an Int
 					//Make it an int
-					TypeCode typeCode = Type.GetTypeCode(_dataType);
+					TypeCode typeCode = Type.GetTypeCode(DataType);
 					if(typeCode != TypeCode.Int16 && 
-					   typeCode != TypeCode.Int32 && 
-					   typeCode != TypeCode.Int64)
+						typeCode != TypeCode.Int32 && 
+						typeCode != TypeCode.Int64)
 					{
-						_dataType = typeof(Int32); 
+						DataType = typeof(Int32); 
 					}
+
+					if (_table != null)
+						_table.Columns.UpdateAutoIncrement(this,true);
+				}
+				else
+				{
+					if (_table != null)
+						_table.Columns.UpdateAutoIncrement(this,false);
 				}
 				_autoIncrement = value;
 			}
@@ -216,14 +248,6 @@ namespace System.Data {
 			}
 		}
 
-		internal void UpdateAutoIncrementValue (long value) 
-		{
-			if(value > _nextAutoIncrementValue) {
-				_nextAutoIncrementValue = value;
-				AutoIncrementValue ();
-			}
-		}
-
 		internal long AutoIncrementValue () 
 		{
 			long currentValue = _nextAutoIncrementValue;
@@ -231,13 +255,9 @@ namespace System.Data {
 			return currentValue;
 		}
 
-		internal bool DataHasBeenSet {
-			get {
-				return dataHasBeenSet;
-			}
-			set {
-				dataHasBeenSet = value;
-			}
+		internal long GetAutoIncrementValue ()
+		{
+			return _nextAutoIncrementValue;
 		}
 
 		[DataCategory ("Data")]
@@ -251,6 +271,9 @@ namespace System.Data {
 					return _caption;
 			}
 			set {
+				if (value == null)
+					value = String.Empty;
+					
 				_caption = value;
 			}
 		}
@@ -273,7 +296,7 @@ namespace System.Data {
 		public string ColumnName
 		{
 			get {
-				return "" + _columnName;
+				return (_columnName == null ? String.Empty : _columnName);
 			}
 			set {
 				//Both are checked after the column is part of the collection
@@ -287,27 +310,36 @@ namespace System.Data {
 		[DataSysDescription ("Indicates the type of data stored in this column.")]
 		[DefaultValue (typeof (string))]
 		[RefreshProperties (RefreshProperties.All)]
+		[TypeConverterAttribute (typeof (ColumnTypeConverter))] 
 		public Type DataType
 		{
 			get {
-				return _dataType;
+				return DataContainer.Type;
 			}
 			set {
-				// check if data already exists can we change the datatype
-				if(DataHasBeenSet == true)
-					throw new ArgumentException("The column already has data stored.");
+				if ( _dataContainer != null ) {
+					if ( value == _dataContainer.Type ) {
+						return;
+					}
 
-				// we want to check that the datatype is supported?
-				TypeCode typeCode = Type.GetTypeCode(value);
+					// check if data already exists can we change the datatype
+					if ( _dataContainer.Capacity > 0 )
+						throw new ArgumentException("The column already has data stored.");
+				}
 				
+				_dataContainer = AbstractDataContainer.CreateInstance(value, this);
+
 				//Check AutoIncrement status, make compatible datatype
 				if(AutoIncrement == true) {
+					// we want to check that the datatype is supported?
+					TypeCode typeCode = Type.GetTypeCode(value);
+					
 					if(typeCode != TypeCode.Int16 &&
 					   typeCode != TypeCode.Int32 &&
-					   typeCode != TypeCode.Int64)
+					   typeCode != TypeCode.Int64) {
 						AutoIncrement = false;
+					}
 				}
-				_dataType = value;
 			}
 		}
 
@@ -319,6 +351,7 @@ namespace System.Data {
 		/// <exception cref="System.ArgumentException"></exception>
 		[DataCategory ("Data")]
 		[DataSysDescription ("Indicates the default column value used when adding new rows to the table.")]
+		[TypeConverterAttribute (typeof (System.Data.DefaultValueTypeConverter))]
 		public object DefaultValue
 		{
 			get {
@@ -360,7 +393,6 @@ namespace System.Data {
 			}
 		}
 
-		[MonoTODO]
 		[DataCategory ("Data")]
 		[DataSysDescription ("Indicates the value that this column computes for each row based on other columns instead of taking user input.")]
 		[DefaultValue ("")]
@@ -371,13 +403,21 @@ namespace System.Data {
 				return expression;
 			}
 			set {
-				//TODO: validation of the expression
-				expression = value;  //Check?
-				
-				if (expression != string.Empty)
+				if (value == null)
+					value = String.Empty;
+					
+		 		if (value != String.Empty) {
+					Parser parser = new Parser ();
+					compiledExpression = parser.Compile (value);
 					ReadOnly = true;
+   				}
+				expression = value;  
 			}
 		}
+		
+		internal IExpression CompiledExpression {
+			get { return compiledExpression; }
+		}
 
 		[Browsable (false)]
 		[DataCategory ("Data")]
@@ -412,6 +452,8 @@ namespace System.Data {
 				return nameSpace;
 			}
 			set {
+				if (value == null)
+					value = String.Empty;
 				nameSpace = value;
 			}
 		}
@@ -443,6 +485,8 @@ namespace System.Data {
 				return prefix;
 			}
 			set {
+				if (value == null)
+					value = String.Empty;
 				prefix = value;
 			}
 		}
@@ -471,7 +515,6 @@ namespace System.Data {
 			}
 		}
 
-		[MonoTODO]
 		[DataCategory ("Data")]
 		[DataSysDescription ("Indicates whether this column should restrict its values in the rows of the table to be unique.")]
 		[DefaultValue (false)]
@@ -482,20 +525,17 @@ namespace System.Data {
 				return unique;
 			}
 			set {
-				//if Table == null then the UniqueConstraint is
-				//created on addition to the collection
-				
-				//FIXME?: need to check if value is the same
-				//because when calling "new UniqueConstraint"
-				//the new object tries to set "column.Unique = True"
-				//which creates an infinite loop.
+				//NOTE: In .NET 1.1 the Unique property
+                                //is left unchanged when it is added
+                                //to a UniqueConstraint
+
 				if(unique != value)
 				{
 				unique = value;
 
 					if( value )
 					{
-						if (Expression != null && Expression != "")
+						if (Expression != null && Expression != String.Empty)
 							throw new ArgumentException("Cannot change Unique property for the expression column.");
 						if( _table != null )
 						{
@@ -533,6 +573,12 @@ namespace System.Data {
 			}
 		}
 
+		internal AbstractDataContainer DataContainer {
+			get {
+				return _dataContainer;
+			}
+		}
+
 		#endregion // Properties
 
 		#region Methods
@@ -576,7 +622,6 @@ namespace System.Data {
 		/// </summary>
 		/// <returns>The Expression value, if the property is set; 
 		/// otherwise, the ColumnName property.</returns>
-		[MonoTODO]
 		public override string ToString()
 		{
 			if (expression != string.Empty)
@@ -585,11 +630,20 @@ namespace System.Data {
 			return ColumnName;
 		}
 
-		[MonoTODO]
 		internal void SetTable(DataTable table) {
-			_table = table; 
-			// this will get called by DataTable 
-			// and DataColumnCollection
+			if(_table!=null) { // serves as double check while adding to a table
+                        	throw new ArgumentException("The column already belongs to a different table");
+                        }
+                        _table = table;
+                        // this will get called by DataTable
+                        // and DataColumnCollection
+                        if(unique) {
+                        	// if the DataColumn is marked as Unique and then
+	                        // added to a DataTable , then a UniqueConstraint
+        	                // should be created
+                	        UniqueConstraint uc = new UniqueConstraint(this);
+                        	_table.Constraints.Add(uc);
+                        }
 		}
 
 		
@@ -617,6 +671,11 @@ namespace System.Data {
 			return true;
 		}
 		
+		internal int CompareValues (int index1, int index2)
+		{
+			return DataContainer.CompareValues(index1, index2);
+		}
+
 		#endregion // Methods
 
 	}
