@@ -21,6 +21,12 @@ namespace System.Data {
 	[DefaultEvent ("CollectionChanged")]
 	public class DataColumnCollection : InternalDataCollectionBase
 	{
+		//This hashtable maps between column name to DataColumn object.
+		private Hashtable columnFromName = new Hashtable();
+		//This ArrayList contains the auto-increment columns names
+		private ArrayList autoIncrement = new ArrayList();
+		//This holds the next index to use for default column name.
+		private int defaultColumnIndex = 1;
 		//table should be the DataTable this DataColumnCollection belongs to.
 		private DataTable parentTable = null;
 
@@ -48,6 +54,11 @@ namespace System.Data {
 		{
 			get
 			{
+				DataColumn dc = columnFromName[name] as DataColumn;
+				
+				if (dc != null)
+					return dc;
+
 				int tmp = IndexOf(name, true);
 				if (tmp == -1)
 					return null;
@@ -66,6 +77,14 @@ namespace System.Data {
 			}
 		}
 
+		internal ArrayList AutoIncrmentColumns 
+		{
+			get
+			{
+				return autoIncrement;
+			}
+		}
+
 		//Add Logic
 		//
 		//Changing Event
@@ -90,27 +109,74 @@ namespace System.Data {
 			CollectionChangeEventArgs e = new CollectionChangeEventArgs(CollectionChangeAction.Add, this);
 			
 			column.SetTable(parentTable);
+			RegisterName(defaultName,column);
 			base.List.Add(column);
 			
+			if (column.AutoIncrement)
+				autoIncrement.Add(defaultName);
+
 			column.SetOrdinal(Count - 1);
 			OnCollectionChanged(e);
 			return column;
 		}
 
+		private void RegisterName(string name, DataColumn column)
+		{
+			if (columnFromName.Contains(name))
+				throw new DuplicateNameException("A DataColumn named '" + name + "' already belongs to this DataTable.");
+
+			columnFromName[name] = column;
+
+			if (name.StartsWith("Column") && name == MakeName(defaultColumnIndex + 1))
+			{
+				do
+				{
+					defaultColumnIndex++;
+				}
+				while (Contains(MakeName(defaultColumnIndex + 1)));
+			}
+		}
+
+		private void UnregisterName(string name)
+		{
+			if (columnFromName.Contains(name))
+				columnFromName.Remove(name);
+
+			if (name.StartsWith("Column") && name == MakeName(defaultColumnIndex - 1))
+			{
+				do
+				{
+					defaultColumnIndex--;
+				}
+				while (!Contains(MakeName(defaultColumnIndex - 1)) && defaultColumnIndex > 1);
+			}
+		}
+
 		private string GetNextDefaultColumnName ()
 		{
-			string defColumnName = "Column1";
-			for (int index = 2; Contains (defColumnName); ++index) {
-				defColumnName = "Column" + index;
+			string defColumnName = MakeName(defaultColumnIndex);
+			for (int index = defaultColumnIndex + 1; Contains(defColumnName); ++index) {
+				defColumnName = MakeName(index);
+				defaultColumnIndex++;
 			}
+			defaultColumnIndex++;
 			return defColumnName;
 		}
 
+		static readonly string[] TenColumns = { "Column0", "Column1", "Column2", "Column3", "Column4", "Column5", "Column6", "Column7", "Column8", "Column9" };
+
+		private string MakeName(int index)
+		{
+			if (index < 10)
+				return TenColumns[index];
+
+			return String.Concat("Column", index.ToString());
+		}
+
 		/// <summary>
 		/// Creates and adds the specified DataColumn object to the DataColumnCollection.
 		/// </summary>
 		/// <param name="column">The DataColumn to add.</param>
-		[MonoTODO]
 		public void Add(DataColumn column)
 		{
 
@@ -121,35 +187,34 @@ namespace System.Data {
 			{
 				column.ColumnName = GetNextDefaultColumnName ();
 			}
-			int tmp = IndexOf(column.ColumnName);
-			// if we found a column with same name we have to check
-			// that it is the same case.
-			// indexof can return a table with different case letters.
-			if (tmp != -1)
-			{
-				if(column.ColumnName == this[tmp].ColumnName)
-					throw new DuplicateNameException("A DataColumn named '" + column.ColumnName + "' already belongs to this DataTable.");
-			}
+
+//			if (Contains(column.ColumnName))
+//				throw new DuplicateNameException("A DataColumn named '" + column.ColumnName + "' already belongs to this DataTable.");
 
 			if (column.Table != null)
-				throw new ArgumentException ("Column '" + column.ColumnName + "' already belongs to another DataTable.");
+				throw new ArgumentException ("Column '" + column.ColumnName + "' already belongs to this or another DataTable.");
 
 			CollectionChangeEventArgs e = new CollectionChangeEventArgs(CollectionChangeAction.Add, this);
 
 			column.SetTable (parentTable);
+			RegisterName(column.ColumnName, column);
 			int ordinal = base.List.Add(column);
 			column.SetOrdinal (ordinal);
 
-			//add constraints if neccesary
-
-			if (column.Unique)
-			{
-				UniqueConstraint uc = new UniqueConstraint(column);
-				parentTable.Constraints.Add(uc);
+			// if table already has rows we need to allocate space 
+			// in the column data container 
+			if ( parentTable.Rows.Count > 0 ) {
+				column.DataContainer.Capacity = parentTable.RecordCache.CurrentCapacity;
 			}
 
-			//TODO: add missing constraints. i.e. Primary/Foreign keys
+			if (column.AutoIncrement) {
+				DataRowCollection rows = column.Table.Rows;
+				for (int i = 0; i < rows.Count; i++)
+					rows [i] [ordinal] = column.AutoIncrementValue ();
+			}
 
+			if (column.AutoIncrement)
+				autoIncrement.Add(column.ColumnName);
 			OnCollectionChanged (e);
 		}
 
@@ -162,7 +227,7 @@ namespace System.Data {
 		{
 			if (columnName == null || columnName == String.Empty)
 			{
-				columnName = GetNextDefaultColumnName ();
+				columnName = GetNextDefaultColumnName();
 			}
 			
 			DataColumn column = new DataColumn(columnName);
@@ -346,7 +411,9 @@ namespace System.Data {
 				}
 
 			}
-					
+			
+			columnFromName.Clear();
+			autoIncrement.Clear();
 			base.List.Clear();
 			OnCollectionChanged(e);
 			return;
@@ -359,6 +426,9 @@ namespace System.Data {
 		/// <returns>true if a column exists with this name; otherwise, false.</returns>
 		public bool Contains(string name)
 		{
+			if (columnFromName.Contains(name))
+				return true;
+			
 			return (IndexOf(name, false) != -1);
 		}
 
@@ -379,6 +449,11 @@ namespace System.Data {
 		/// <returns>The zero-based index of the column with the specified name, or -1 if the column doesn't exist in the collection.</returns>
 		public int IndexOf(string columnName)
 		{
+			DataColumn dc = columnFromName[columnName] as DataColumn;
+				
+			if (dc != null)
+				return IndexOf(dc);
+
 			return IndexOf(columnName, false);
 		}
 
@@ -414,14 +489,17 @@ namespace System.Data {
 		/// <param name="column">The DataColumn to remove.</param>
 		public void Remove(DataColumn column)
 		{
-			if (IndexOf (column) == -1)
-				throw new ArgumentException ("Cannot remove a column that doesn't belong to this table.");
+			if (column == null)
+				throw new ArgumentNullException ("column", "'column' argument cannot be null.");
 
+			if (!Contains(column.ColumnName))
+				throw new ArgumentException ("Cannot remove a column that doesn't belong to this table.");
 			//TODO: can remove first with exceptions
 			//and OnChanging Event
 			CollectionChangeEventArgs e = new CollectionChangeEventArgs(CollectionChangeAction.Remove, this);
 			
 			int ordinal = column.Ordinal;
+			UnregisterName(column.ColumnName);
 			base.List.Remove(column);
 			
 			//Update the ordinals
@@ -430,6 +508,12 @@ namespace System.Data {
 				this[i].SetOrdinal( i );
 			}
 			
+			if (parentTable != null)
+				parentTable.OnRemoveColumn(column);
+
+			if (column.AutoIncrement)
+				autoIncrement.Remove(column.ColumnName);
+
 			OnCollectionChanged(e);
 			return;
 		}
@@ -443,9 +527,8 @@ namespace System.Data {
 			DataColumn column = this[name];
 			
 			if (column == null)
-				throw new ArgumentException ("Column '" + name + "' does not belong to table test_table.");
-
-			Remove( column );
+				throw new ArgumentException ("Column '" + name + "' does not belong to table " + ( parentTable == null ? "" : parentTable.TableName ) + ".");
+			Remove(column);
 		}
 
 		/// <summary>
@@ -458,7 +541,7 @@ namespace System.Data {
 				throw new IndexOutOfRangeException ("Cannot find column " + index + ".");
 
 			DataColumn column = this[index];
-			Remove( column );
+			Remove(column);
 		}
 
 
@@ -467,21 +550,34 @@ namespace System.Data {
 		/// </summary>
 		private bool CaseSensitiveContains(string columnName)
 		{
-			
 			DataColumn column = this[columnName];
 			
 			if (column != null)
-				return string.Compare (column.ColumnName, columnName, false) == 0; 
+				return string.Compare(column.ColumnName, columnName, false) == 0; 
 
 			return false;
 		}
 
+		internal void UpdateAutoIncrement(DataColumn col,bool isAutoIncrement)
+		{
+			if (isAutoIncrement)
+			{
+				if (!autoIncrement.Contains(col.ColumnName))
+					autoIncrement.Add(col.ColumnName);
+			}
+			else
+			{
+				if (autoIncrement.Contains(col.ColumnName))
+					autoIncrement.Remove(col.ColumnName);
+			}
+		}
+
 		private int IndexOf (string name, bool error)
 		{
 			int count = 0, match = -1;
-			for (int i = 0; i < list.Count; i++)
+			for (int i = 0; i < List.Count; i++)
 			{
-				String name2 = ((DataColumn) list[i]).ColumnName;
+				String name2 = ((DataColumn) List[i]).ColumnName;
 				if (String.Compare (name, name2, true) == 0)
 				{
 					if (String.Compare (name, name2, false) == 0)
@@ -497,10 +593,14 @@ namespace System.Data {
 			return -1;
 		}
 		
+		#region Events
 
 		/// <summary>
 		/// Occurs when the columns collection changes, either by adding or removing a column.
 		/// </summary>
+                [ResDescriptionAttribute ("Occurs whenever this collection's membership changes.")] 
 		public event CollectionChangeEventHandler CollectionChanged;
+
+		#endregion 
 	}
 }
