@@ -22,6 +22,7 @@ namespace System.Data {
 	/// </summary>
 	[Editor]
 	[DefaultEvent ("CollectionChanged")]
+	[DefaultProperty ("Table")]
 	[Serializable]
 	public abstract class DataRelationCollection : InternalDataCollectionBase
 	{
@@ -46,9 +47,8 @@ namespace System.Data {
 			public override DataRelation this [string name]
 			{
 				get {
-					foreach (DataRelation dataRelation in List)
-						if (dataRelation.RelationName == name) return dataRelation;
-					return null;
+					int index = IndexOf (name, true);
+					return index < 0 ? null : (DataRelation) List[index];
 				}
 			}
 
@@ -58,7 +58,11 @@ namespace System.Data {
 			public override DataRelation this [int index]
 			{
 				get {
-					return List [index] as DataRelation;
+					try {
+						return List [index] as DataRelation;
+					} catch (ArgumentOutOfRangeException e) {
+						throw new IndexOutOfRangeException (String.Format ("Cannot find relation {0}.", index));
+					}
 				}
 			}
 
@@ -179,9 +183,8 @@ namespace System.Data {
 			public override DataRelation this [string name]
 			{
 				get {
-					foreach (DataRelation dataRelation in List)
-						if (dataRelation.RelationName == name) return dataRelation;
-					return null;
+					int index = IndexOf (name, true);
+					return index < 0 ? null : (DataRelation) List[index];
 				}
 			}
 
@@ -191,7 +194,11 @@ namespace System.Data {
 			public override DataRelation this [int index]
 			{
 				get {
-					return List [index] as DataRelation;
+					try {
+						return List [index] as DataRelation;
+					} catch (ArgumentOutOfRangeException e) {
+						throw new IndexOutOfRangeException (String.Format ("Cannot find relation {0}.", index));
+					}
 				}
 			}
 
@@ -246,8 +253,9 @@ namespace System.Data {
 		#region Add Methods
 		private string GetNextDefaultRelationName ()
 		{
-			string defRelationName = "Relation";
-			for (int index = 1; Contains (defRelationName); ++index) {
+			int index = 1;
+			string defRelationName = "Relation" +index;
+			for (; Contains (defRelationName); ++index) {
 				defRelationName = "Relation" + index;
 			}
 			return defRelationName;
@@ -398,15 +406,21 @@ namespace System.Data {
 				//TODO: Issue a good exception message.
 				throw new ArgumentNullException();
 			}
-			else if(List.IndexOf(relation) != -1)
+			if(List.IndexOf(relation) != -1)
 			{
 				//TODO: Issue a good exception message.
 				throw new ArgumentException();
 			}
-			else if(List.Contains(relation.RelationName))
+
+			// check if the collection has a relation with the same name.
+			int tmp = IndexOf(relation.RelationName);
+			// if we found a relation with same name we have to check
+			// that it is the same case.
+			// indexof can return a table with different case letters.
+			if (tmp != -1)
 			{
-				//TODO: Issue a good exception message.
-				throw new DuplicateNameException("A Relation named " + relation.RelationName + " already belongs to this DataSet.");
+				if(relation.RelationName == this[tmp].RelationName)
+					throw new DuplicateNameException("A DataRelation named '" + relation.RelationName + "' already belongs to this DataSet.");
 			}
 			List.Add(relation);
 		}
@@ -424,7 +438,21 @@ namespace System.Data {
 		{
 			if (relation == null || !GetDataSet().Equals(relation.DataSet))
 				return false;
-			return true;
+
+			// check if the relation doesnot belong to this collection
+                        int tmp = IndexOf(relation.RelationName);
+                        // if we found a relation with same name we have to check
+                        // that it is the same case.
+                        // indexof can return a table with different case letters.
+                        if (tmp != -1) {
+                               if(relation.RelationName != this[tmp].RelationName)
+                                        return false;
+                        }
+                        else {
+                                return false;
+                        }                                       
+                                                                                        
+                        return true;
 		}
 
 		public virtual void Clear()
@@ -434,7 +462,7 @@ namespace System.Data {
 
 		public virtual bool Contains(string name)
 		{
-			return IndexOf(name) != -1;
+			return (-1 != IndexOf (name, false));
 		}
 
 		private CollectionChangeEventArgs CreateCollectionChangeEvent (CollectionChangeAction action)
@@ -451,7 +479,28 @@ namespace System.Data {
 
 		public virtual int IndexOf(string relationName)
 		{
-			return List.IndexOf(this[relationName]);
+			return IndexOf(relationName, false);
+		}
+
+		private int IndexOf (string name, bool error)
+		{
+			int count = 0, match = -1;
+			for (int i = 0; i < List.Count; i++)
+			{
+				String name2 = ((DataRelation) List[i]).RelationName;
+				if (String.Compare (name, name2, true) == 0)
+				{
+					if (String.Compare (name, name2, false) == 0)
+						return i;
+					match = i;
+					count++;
+				}
+			}
+			if (count == 1)
+				return match;
+			if (count > 1 && error)
+				throw new ArgumentException ("There is no match for the name in the same case and there are multiple matches in different case.");
+			return -1;
 		}
 
 		protected virtual void OnCollectionChanged (CollectionChangeEventArgs ccevent)
@@ -468,6 +517,13 @@ namespace System.Data {
 
 		public void Remove (DataRelation relation)
 		{
+			if (relation == null)
+				throw new ArgumentNullException("Relation specified is null");
+
+			 // check if the list doesnot contains this relation.
+                        if (!(List.Contains(relation)))
+                                throw new ArgumentException("Relation doesnot belong to this Collection.");
+
 			RemoveCore (relation);
 			List.Remove (relation);
 			string name = "Relation" + index;
@@ -483,6 +539,8 @@ namespace System.Data {
 
 		public void RemoveAt (int index)
 		{
+			if (( index < 0 ) || (index >=List.Count))
+                                throw new ArgumentException("There is no row at position "+index+".");
 			Remove(this[index]);
 		}
 
@@ -492,7 +550,11 @@ namespace System.Data {
 			// TODO: What have to be done?
 		}
 
+		#region Events
+
 		[ResDescriptionAttribute ("Occurs whenever this collection's membership changes.")]
 		public event CollectionChangeEventHandler CollectionChanged;
+
+		#endregion
 	}
 }
