@@ -41,7 +41,7 @@ namespace System.Data
 				if (index >= Count)
 					throw new IndexOutOfRangeException ("There is no row at position " + index + ".");
 
-				return (DataRow) list[index]; 
+				return (DataRow) List[index]; 
 			}
 		}
 
@@ -50,7 +50,7 @@ namespace System.Data
 		/// </summary>
 		protected override ArrayList List 
 		{
-			get { return list; }
+			get { return base.List; }
 		}		
 
 		/// <summary>
@@ -64,8 +64,9 @@ namespace System.Data
 
 			if (row.Table != this.table)
 				throw new ArgumentException ("This row already belongs to another table.");
-
-			if (list.IndexOf(row) != -1)
+			
+			// If row id is not -1, we know that it is in the collection.
+			if (row.RowID != -1)
 				throw new ArgumentException ("This row already belongs to this table.");
 			
 
@@ -74,7 +75,9 @@ namespace System.Data
 				ValidateDataRowInternal(row);
 			
 			row.HasParentCollection = true;
-			list.Add (row);
+			List.Add (row);
+			// Set the row id.
+			row.RowID = List.Count - 1;
 			row.AttachRow ();
 			row.Table.ChangedDataRow (row, DataRowAction.Add);
 		}
@@ -84,6 +87,8 @@ namespace System.Data
 		/// </summary>
 		public virtual DataRow Add (object[] values) 
 		{
+			if (values.Length > table.Columns.Count)
+				throw new ArgumentException ("The array is larger than the number of columns in the table.");
 			DataRow row = table.NewRow ();
 			row.ItemArray = values;
 			Add (row);
@@ -104,12 +109,16 @@ namespace System.Data
 						if (c is ForeignKeyConstraint)
 						{
 							if (((ForeignKeyConstraint) c).RelatedTable.Equals(this.table))
-								throw new InvalidConstraintException("Cannot clear table Parent because ForeignKeyConstraint " + c.ConstraintName + " enforces Child.");
+#if NET_1_1
+								throw new InvalidConstraintException (String.Format ("Cannot clear table Parent because ForeignKeyConstraint {0} enforces Child.", c.ConstraintName));
+#else
+								throw new ArgumentException (String.Format ("Cannot clear table Parent because ForeignKeyConstraint {0} enforces Child.", c.ConstraintName));
+#endif
 						}
 					}
 				}
 			}
-			list.Clear ();
+			List.Clear ();
 		}
 
 		/// <summary>
@@ -137,93 +146,107 @@ namespace System.Data
 		/// <summary>
 		/// Gets the row specified by the primary key value.
 		/// </summary>
-		[MonoTODO]
 		public DataRow Find (object key) 
 		{
 			if (table.PrimaryKey.Length == 0)
 				throw new MissingPrimaryKeyException ("Table doesn't have a primary key.");
 			if (table.PrimaryKey.Length > 1)
-				throw new ArgumentException ("Expecting " + table.PrimaryKey.Length + 
-							     " value(s) for the key being indexed, but received 1 value(s).");
+				throw new ArgumentException ("Expecting " + table.PrimaryKey.Length +" value(s) for the key being indexed, but received 1 value(s).");
 
-			string primColumnName = table.PrimaryKey [0].ColumnName;
-			Type coltype = null;
-			object newKey = null;
-			
-			foreach (DataRow row in this) {
-				
-				if (row.RowState != DataRowState.Deleted)
-				{
-					object primValue = row [primColumnName];
-					if (key == null) 
-					{
-						if (primValue == null)
-							return row;
-						else 
-							continue;
+			if (key == null)
+#if NET_1_1
+				return null;
+#else
+				throw new ArgumentException("Expecting 1 value(s) for the key being indexed, but received 0 value(s).");
+#endif
+
+			DataColumn primaryKey = table.PrimaryKey[0];
+			Index primaryKeyIndex = table.GetIndexByColumns(table.PrimaryKey);
+			int tmpRecord = table.RecordCache.NewRecord();
+			try {
+				primaryKey.DataContainer[tmpRecord] = key;
+
+				// if we can search through index
+				if (primaryKeyIndex != null) {
+					// get the child rows from the index
+					Node node = primaryKeyIndex.FindSimple(tmpRecord,1,true);
+					if (node != null) {
+						return node.Row;
 					}
-				       
-					newKey = Convert.ChangeType (key, Type.GetTypeCode(primValue.GetType ()));
-
-					if (primValue.Equals (newKey))
-						return row;
 				}
+			
+				//loop through all collection rows			
+				foreach (DataRow row in this) {
+					if (row.RowState != DataRowState.Deleted) {
+						int index = row.IndexFromVersion(DataRowVersion.Default); 
+						if (primaryKey.DataContainer.CompareValues(index, tmpRecord) == 0) {
+							return row;
+						}
+					}
+				}
+				return null;
+			}
+			finally {
+				table.RecordCache.DisposeRecord(tmpRecord);
 			}
-						
-			// FIXME: is the correct value null?
-			return null;
 		}
 
 		/// <summary>
 		/// Gets the row containing the specified primary key values.
 		/// </summary>
-		[MonoTODO]
 		public DataRow Find (object[] keys) 
 		{
 			if (table.PrimaryKey.Length == 0)
 				throw new MissingPrimaryKeyException ("Table doesn't have a primary key.");
 
-			string  [] primColumnNames = new string [table.PrimaryKey.Length];
-			
-			for (int i = 0; i < primColumnNames.Length; i++)
-				primColumnNames [i] = table.PrimaryKey [i].ColumnName;
+			if (keys == null)
+				throw new ArgumentException ("Expecting " + table.PrimaryKey.Length +" value(s) for the key being indexed, but received 0 value(s).");
+			if (table.PrimaryKey.Length != keys.Length)
+				throw new ArgumentException ("Expecting " + table.PrimaryKey.Length +" value(s) for the key being indexed, but received " + keys.Length +  " value(s).");
+                                                                                                    
+			DataColumn[] primaryKey = table.PrimaryKey;
+			int tmpRecord = table.RecordCache.NewRecord();
+			try {
+				int numColumn = keys.Length;
+				for (int i = 0; i < numColumn; i++) {
+					// according to MSDN: the DataType value for both columns must be identical.
+					primaryKey[i].DataContainer[tmpRecord] = keys[i];
+				}
+				return Find(tmpRecord,numColumn);
+			}
+			finally {
+				table.RecordCache.DisposeRecord(tmpRecord);
+			}
+		}
 
-			Type coltype = null;
-			object newKey = null;
-			
+		internal DataRow Find(int index, int length)
+		{
+			DataColumn[] primaryKey = table.PrimaryKey;
+			Index primaryKeyIndex = table.GetIndexByColumns(primaryKey);
+			// if we can search through index
+			if (primaryKeyIndex != null) {
+				// get the child rows from the index
+				Node node = primaryKeyIndex.FindSimple(index,length,true);
+				if ( node != null ) {
+					return node.Row;
+				}
+			}
+		
+			//loop through all collection rows			
 			foreach (DataRow row in this) {
-				
-				if (row.RowState != DataRowState.Deleted)
-				{
-					bool eq = true;
-					for (int i = 0; i < keys.Length; i++) 
-					{
-					
-						object primValue = row [primColumnNames [i]];
-						object keyValue = keys [i];
-						if (keyValue == null) 
-						{
-							if (primValue == null)
-								return row;
-							else 
-								continue;
+				if (row.RowState != DataRowState.Deleted) {
+					int rowIndex = row.IndexFromVersion(DataRowVersion.Default);
+					bool match = true;
+					for (int columnCnt = 0; columnCnt < length; ++columnCnt) { 
+						if (primaryKey[columnCnt].DataContainer.CompareValues(rowIndex, index) != 0) {
+							match = false;
 						}
-								       
-						newKey = Convert.ChangeType (keyValue, Type.GetTypeCode(primValue.GetType ()));
-
-						if (!primValue.Equals (newKey)) 
-						{
-							eq = false;
-							break;
-						}						
 					}
-
-					if (eq)
+					if ( match ) {
 						return row;
+					}
 				}
 			}
-						
-			// FIXME: is the correct value null?
 			return null;
 		}
 
@@ -234,11 +257,36 @@ namespace System.Data
 		{
 			if (pos < 0)
 				throw new IndexOutOfRangeException ("The row insert position " + pos + " is invalid.");
+			
+			if (row == null)
+				throw new ArgumentNullException("row", "'row' argument cannot be null.");
+	
+			if (row.Table != this.table)
+				throw new ArgumentException ("This row already belongs to another table.");
+
+			// If row id is not -1, we know that it is in the collection.
+			if (row.RowID != -1)
+				throw new ArgumentException ("This row already belongs to this table.");
+			
+			if ((table.DataSet == null || table.DataSet.EnforceConstraints) && !table._duringDataLoad)
+				// we have to check that the new row doesn't colide with existing row
+				ValidateDataRowInternal(row);
 				
-			if (pos >= list.Count)
-				list.Add (row);
-			else
-				list.Insert (pos, row);
+			if (pos >= List.Count) {
+				row.RowID = List.Count;
+				List.Add (row);
+			}
+			else {
+				List.Insert (pos, row);
+				row.RowID = pos;
+				for (int i = pos+1; i < List.Count; i++) {
+        	                        ((DataRow)List [i]).RowID = i;
+	                        }
+			}
+				
+			row.HasParentCollection = true;
+			row.AttachRow ();
+			row.Table.ChangedDataRow (row, DataRowAction.Add);
 		}
 
 		/// <summary>
@@ -248,11 +296,11 @@ namespace System.Data
 			if (row == null) {
 				throw new IndexOutOfRangeException ("The given datarow is not in the current DataRowCollection.");
 			}
-			int index = list.IndexOf(row);
+			int index = List.IndexOf(row);
 			if (index < 0) {
 				throw new IndexOutOfRangeException ("The given datarow is not in the current DataRowCollection.");
 			}
-			list.RemoveAt(index);
+			List.RemoveAt(index);
 		}
 
 		/// <summary>
@@ -262,7 +310,7 @@ namespace System.Data
 		{
 			if (row == null)
 				throw new IndexOutOfRangeException ("The given datarow is not in the current DataRowCollection.");
-			int index = list.IndexOf(row);
+			int index = List.IndexOf(row);
 			if (index < 0)
 				throw new IndexOutOfRangeException ("The given datarow is not in the current DataRowCollection.");
 			row.Delete();
@@ -277,11 +325,14 @@ namespace System.Data
 		/// </summary>
 		public void RemoveAt (int index) 
 		{			
-			if (index < 0 || index >= list.Count)
+			if (index < 0 || index >= List.Count)
 				throw new IndexOutOfRangeException ("There is no row at position " + index + ".");
-			DataRow row = (DataRow)list [index];
+			DataRow row = (DataRow)List [index];
 			row.Delete();
-			row.AcceptChanges();
+			// if the row was in added state it will be in Detached state after the
+			// delete operation, so we have to check it.
+			if (row.RowState != DataRowState.Detached)
+				row.AcceptChanges();
 		}
 
 		///<summary>
@@ -292,18 +343,55 @@ namespace System.Data
 		internal void ValidateDataRowInternal(DataRow row)
 		{
 			//first check for null violations.
+			row._nullConstraintViolation = true;
 			row.CheckNullConstraints();
-			//FIXME: this validates constraints in the order they appear
-			//in the collection. Most probably we need to do it in a 
-			//specific order like unique/primary keys first, then Foreignkeys, etc
-			foreach(Constraint constraint in table.Constraints)
-			{
-				constraint.AssertConstraint(row);
+			// This validates constraints in the specific order : 
+			// first unique/primary keys first, then Foreignkeys, etc
+			ArrayList uniqueConstraintsDone = new ArrayList();
+			ArrayList foreignKeyConstraintsDone = new ArrayList();
+			try {
+				foreach(Constraint constraint in table.Constraints.UniqueConstraints) {
+					constraint.AssertConstraint(row);
+					uniqueConstraintsDone.Add(constraint);
+				}
+			
+				foreach(Constraint constraint in table.Constraints.ForeignKeyConstraints) {
+					constraint.AssertConstraint(row);
+					foreignKeyConstraintsDone.Add(constraint);
+				}
+			}
+			// if one of the AssertConstraint failed - we need to "rollback" all the changes
+			// caused by AssertCoinstraint calls already succeeded
+			catch(ConstraintException e) {
+				RollbackAsserts(row,foreignKeyConstraintsDone,uniqueConstraintsDone);
+				throw e;
+			}
+			catch(InvalidConstraintException e) {	
+				RollbackAsserts(row,foreignKeyConstraintsDone,uniqueConstraintsDone);
+				throw e;
+			}
+		}
+
+		private void RollbackAsserts(DataRow row,ICollection foreignKeyConstraintsDone,
+			ICollection uniqueConstraintsDone)
+		{
+			// if any of constraints assert failed - 
+			// we have to rollback all the asserts scceeded
+			// on order reverse to thier original execution
+			foreach(Constraint constraint in foreignKeyConstraintsDone) {
+				constraint.RollbackAssert(row);
 			}
 
+			foreach(Constraint constraint in uniqueConstraintsDone) {
+				constraint.RollbackAssert(row);
+			}
+		}
+
+		internal void onColumnRemoved(int columnIndex)
+		{
+			foreach(DataRow row in List) {
+				row.onColumnRemoved(columnIndex);
+			}
 		}
-		
 	}
-
-
 }
