@@ -16,12 +16,15 @@
 //
 
 using System;
+using System.Data.Common;
 using System.Collections;
 using System.ComponentModel;
 using System.Globalization;
 using System.IO;
 using System.Runtime.Serialization;
 using System.Xml;
+using System.Xml.Schema;
+using Mono.Data.SqlExpressions;
 
 namespace System.Data {
 	//[Designer]
@@ -29,9 +32,12 @@ namespace System.Data {
 	[DefaultEvent ("RowChanging")]
 	[DefaultProperty ("TableName")]
 	[DesignTimeVisible (false)]
+	[EditorAttribute ("Microsoft.VSDesigner.Data.Design.DataTableEditor, "+ Consts.AssemblyMicrosoft_VSDesigner, "System.Drawing.Design.UITypeEditor, "+ Consts.AssemblySystem_Drawing )]
 	[Serializable]
 	public class DataTable : MarshalByValueComponent, IListSource, ISupportInitialize, ISerializable 
 	{
+		#region Fields
+
 		internal DataSet dataSet;   
 		
 		private bool _caseSensitive;
@@ -55,18 +61,38 @@ namespace System.Data {
 		private bool _containsListCollection;
 		private string _encodedTableName;
 		internal bool _duringDataLoad;
+		internal bool _nullConstraintViolationDuringDataLoad;
 		private bool dataSetPrevEnforceConstraints;
-
+		private bool dataTablePrevEnforceConstraints;
+		private bool enforceConstraints = true;
+		private DataRowBuilder _rowBuilder;
+		private ArrayList _indexes;
+		private RecordCache _recordCache;
 
 		
 		// If CaseSensitive property is changed once it does not anymore follow owner DataSet's 
 		// CaseSensitive property. So when you lost you virginity it's gone for ever
 		private bool _virginCaseSensitive = true;
+		
+		 /// <summary>
+                 /// An enum variable indicating whether BeginInit() or EndInit() is called.
+                /// Delegate to call a function which performs cleanup after EndInit() is called
+                /// </summary>
+                internal enum initStatus { NotInitialized, BeginInit, EndInit };
+	        private initStatus _initStatus;
+        
+		#endregion //Fields
+		
+		private delegate void PostEndInit();
+                
+		internal initStatus InitStatus {
+                        get{ return( _initStatus ); }
+                }
+	
 
 		/// <summary>
 		/// Initializes a new instance of the DataTable class with no arguments.
 		/// </summary>
-		
 		public DataTable () 
 		{
 			dataSet = null;
@@ -80,6 +106,8 @@ namespace System.Data {
 			_primaryKey = null;
 			_site = null;
 			_rows = new DataRowCollection (this);
+			_indexes = new ArrayList();
+			_recordCache = new RecordCache(this);
 			
 			//LAMESPEC: spec says 25 impl does 50
 			_minimumCapacity = 50;
@@ -87,7 +115,6 @@ namespace System.Data {
 			_childRelations = new DataRelationCollection.DataTableRelationCollection (this);
 			_parentRelations = new DataRelationCollection.DataTableRelationCollection (this);
 
-		
 			_defaultView = new DataView(this);
 		}
 
@@ -106,12 +133,29 @@ namespace System.Data {
 		protected DataTable (SerializationInfo info, StreamingContext context)
 			: this () 
 		{
-			//
-			// TODO: Add constructor logic here
-			//
+			string schema = info.GetString ("XmlSchema");
+			string data = info.GetString ("XmlDiffGram");
+			
+			DataSet ds = new DataSet ();
+			ds.ReadXmlSchema (new StringReader (schema));
+			ds.Tables [0].CopyProperties (this);
+			ds = new DataSet ();
+			ds.Tables.Add (this);
+			ds.ReadXml (new StringReader (data), XmlReadMode.IgnoreSchema);
+			ds.Tables.Remove (this);
+/* keeping for a while. With the change above, we shouldn't have to consider 
+ * DataTable mode in schema inference/read.
+			XmlSchemaMapper mapper = new XmlSchemaMapper (this);
+			XmlTextReader xtr = new XmlTextReader(new StringReader (schema));
+			mapper.Read (xtr);
+			
+			XmlDiffLoader loader = new XmlDiffLoader (this);
+			xtr = new XmlTextReader(new StringReader (data));
+			loader.Load (xtr);
+*/
 		}
 
-#if NET_1_2
+#if NET_2_0
 		public DataTable (string tableName, string tbNamespace)
 			: this (tableName)
 		{
@@ -124,8 +168,16 @@ namespace System.Data {
 		/// </summary>
 		[DataSysDescription ("Indicates whether comparing strings within the table is case sensitive.")]	
 		public bool CaseSensitive {
-			get { return _caseSensitive; }
-			set { 
+			get { 
+				if (_virginCaseSensitive && dataSet != null)
+					return dataSet.CaseSensitive; 
+				else
+					return _caseSensitive;
+				}
+			set {
+				if (_childRelations.Count > 0 || _parentRelations.Count > 0) {
+					throw new ArgumentException ("Cannot change CaseSensitive or Locale property. This change would lead to at least one DataRelation or Constraint to have different Locale or CaseSensitive settings between its related tables.");
+				}
 				_virginCaseSensitive = false;
 				_caseSensitive = value; 
 			}
@@ -136,6 +188,10 @@ namespace System.Data {
 			set { _virginCaseSensitive = value; }
 		}
 
+		internal ArrayList Indexes{
+			get { return _indexes; }
+		}
+
 		internal void ChangedDataColumn (DataRow dr, DataColumn dc, object pv) 
 		{
 			DataColumnChangeEventArgs e = new DataColumnChangeEventArgs (dr, dc, pv);
@@ -236,7 +292,7 @@ namespace System.Data {
 		[DataSysDescription ("The expression used to compute the data-bound value of this row.")]	
 		[DefaultValue ("")]
 		public string DisplayExpression {
-			get { return "" + _displayExpression; }
+			get { return _displayExpression == null ? "" : _displayExpression; }
 			set { _displayExpression = value; }
 		}
 
@@ -287,6 +343,9 @@ namespace System.Data {
 				return CultureInfo.CurrentCulture;
 			}
 			set { 
+				if (_childRelations.Count > 0 || _parentRelations.Count > 0) {
+					throw new ArgumentException ("Cannot change CaseSensitive or Locale property. This change would lead to at least one DataRelation or Constraint to have different Locale or CaseSensitive settings between its related tables.");
+				}
 				if (_locale == null || !_locale.Equals(value))
 					_locale = value; 
 			}
@@ -310,7 +369,7 @@ namespace System.Data {
 		[DataCategory ("Data")]
 		[DataSysDescription ("Indicates the XML uri namespace for the elements contained in this table.")]
 		public string Namespace {
-			get { return "" + _nameSpace; }
+			get { return _nameSpace == null ? "" : _nameSpace; }
 			set { _nameSpace = value; }
 		}
 
@@ -335,8 +394,15 @@ namespace System.Data {
 		[DataSysDescription ("Indicates the Prefix of the namespace used for this table in XML representation.")]
 		[DefaultValue ("")]
 		public string Prefix {
-			get { return "" + _prefix; }
-			set { _prefix = value; }
+			get { return _prefix == null ? "" : _prefix; }
+			set {
+				// Prefix cannot contain any special characters other than '_' and ':'
+				for (int i = 0; i < value.Length; i++) {
+					if (!(Char.IsLetterOrDigit (value [i])) && (value [i] != '_') && (value [i] != ':'))
+						throw new DataException ("Prefix '" + value + "' is not valid, because it contains special characters.");
+				}
+				_prefix = value;
+			}
 		}
 
 		/// <summary>
@@ -345,6 +411,8 @@ namespace System.Data {
 		/// </summary>
 		[DataCategory ("Data")]
 		[DataSysDescription ("Indicates the column(s) that represent the primary key for this table.")]
+		[EditorAttribute ("Microsoft.VSDesigner.Data.Design.PrimaryKeyEditor, "+ Consts.AssemblyMicrosoft_VSDesigner, "System.Drawing.Design.UITypeEditor, "+ Consts.AssemblySystem_Drawing )]
+		[TypeConverterAttribute ("System.Data.PrimaryKeyTypeConverter, System.Data, Version=1.0.5000.0, Culture=neutral, PublicKeyToken=b77a5c561934e089")]
 		public DataColumn[] PrimaryKey {
 			get {
 				UniqueConstraint uc = UniqueConstraint.GetPrimaryKeyConstraint( Constraints);
@@ -359,23 +427,19 @@ namespace System.Data {
 					return;
 
 				// remove PK Constraint
-				if(oldPKConstraint != null)
-				{
+				if(oldPKConstraint != null) {
 					Constraints.Remove(oldPKConstraint);
 				}
 				
-				if (value != null)
-				{
+				if (value != null) {
 					//Does constraint exist for these columns
 					UniqueConstraint uc = UniqueConstraint.GetUniqueConstraintForColumnSet(
 						this.Constraints, (DataColumn[]) value);
 				
 					//if constraint doesn't exist for columns
 					//create new unique primary key constraint
-					if (null == uc) 
-					{
-						foreach (DataColumn Col in (DataColumn[]) value) 
-						{
+					if (null == uc) {
+						foreach (DataColumn Col in (DataColumn[]) value) {
 
 							if (Col.Table == null)
 								break;
@@ -389,10 +453,26 @@ namespace System.Data {
 					
 						Constraints.Add (uc);
 					}
-					else 
-					{ //set existing constraint as the new primary key
+					else { 
+						//set existing constraint as the new primary key
 						UniqueConstraint.SetAsPrimaryKey(this.Constraints, uc);
 					}
+
+					// if we really supplied some columns fo primary key - 
+					// rebuild indexes fo all foreign key constraints
+					if(value.Length > 0) {
+						foreach(ForeignKeyConstraint constraint in this.Constraints.ForeignKeyConstraints){
+							constraint.AssertConstraint();
+						}
+					}
+				}
+				else {
+					// if primary key is null now - drop all the indexes for foreign key constraints
+					foreach(ForeignKeyConstraint constraint in this.Constraints.ForeignKeyConstraints){
+						Index index = constraint.Index;
+						constraint.Index = null;
+						this.DropIndex(index);
+					}
 				}
 				
 			}
@@ -426,7 +506,7 @@ namespace System.Data {
 		[DefaultValue ("")]	
 		[RefreshProperties (RefreshProperties.All)]
 		public string TableName {
-			get { return "" + _tableName; }
+			get { return _tableName == null ? "" : _tableName; }
 			set { _tableName = value; }
 		}
 		
@@ -436,6 +516,80 @@ namespace System.Data {
 				return false;
 			}
 		}
+		
+		internal RecordCache RecordCache {
+			get {
+				return _recordCache;
+			}
+		}
+		
+		private bool EnforceConstraints {
+			get { return enforceConstraints; }
+			set {
+				if (value != enforceConstraints) {
+					if (value) {
+						// first assert all unique constraints
+						foreach (UniqueConstraint uc in this.Constraints.UniqueConstraints)
+						uc.AssertConstraint ();
+						// then assert all foreign keys
+						foreach (ForeignKeyConstraint fk in this.Constraints.ForeignKeyConstraints)
+							fk.AssertConstraint ();
+					}
+					enforceConstraints = value;
+				}
+			}
+		}
+
+		internal bool RowsExist(DataColumn[] columns, DataColumn[] relatedColumns,DataRow row)
+		{
+			int curIndex = row.IndexFromVersion(DataRowVersion.Default);
+			int tmpRecord = RecordCache.NewRecord();
+
+			try {
+				for (int i = 0; i < relatedColumns.Length; i++) {
+					// according to MSDN: the DataType value for both columns must be identical.
+					columns[i].DataContainer.CopyValue(relatedColumns[i].DataContainer, curIndex, tmpRecord);
+				}
+
+				return RowsExist(columns, tmpRecord, relatedColumns.Length);
+			}
+			finally {
+				RecordCache.DisposeRecord(tmpRecord);
+			}
+		}
+
+		bool RowsExist(DataColumn[] columns, int index, int length)
+		{
+			bool rowsExist = false;
+			Index indx = this.GetIndexByColumns (columns);
+
+			if (indx != null) { // lookup for a row in index			
+				rowsExist = (indx.FindSimple (index, length, false) != null);
+			} 
+			
+			if(indx == null || rowsExist == false) { 
+				// no index or rowExist= false, we have to perform full-table scan
+ 				// check that there is a parent for this row.
+				foreach (DataRow thisRow in this.Rows) {
+					if (thisRow.RowState != DataRowState.Deleted) {
+						bool match = true;
+						// check if the values in the columns are equal
+						int thisIndex = thisRow.IndexFromVersion(DataRowVersion.Current);
+						foreach (DataColumn column in columns) {
+							if (column.DataContainer.CompareValues(thisIndex, index) != 0) {
+								match = false;
+								break;
+							}	
+						}
+						if (match) {// there is a row with columns values equals to those supplied.
+							rowsExist = true;
+							break;
+						}
+					}
+				}				
+			}
+			return rowsExist;
+		}
 
 		/// <summary>
 		/// Commits all the changes made to this table since the 
@@ -466,8 +620,9 @@ namespace System.Data {
 		/// on a form or used by another component. The initialization
 		/// occurs at runtime.
 		/// </summary>
-		public void BeginInit () 
+		public virtual void BeginInit () 
 		{
+			_initStatus = initStatus.BeginInit;
 		}
 
 		/// <summary>
@@ -482,6 +637,7 @@ namespace System.Data {
 				//duringDataLoad is important to EndLoadData and
 				//for not throwing unexpected exceptions.
 				this._duringDataLoad = true;
+				this._nullConstraintViolationDuringDataLoad = false;
 			
 				if (this.dataSet != null)
 				{
@@ -490,6 +646,11 @@ namespace System.Data {
 					this.dataSetPrevEnforceConstraints = this.dataSet.EnforceConstraints;
 					this.dataSet.EnforceConstraints = false;
 				}
+				else {
+					//if table does not belong to any data set use EnforceConstraints of the table
+					this.dataTablePrevEnforceConstraints = this.EnforceConstraints;
+					this.EnforceConstraints = false;
+				}
 			}
 			return;
 		}
@@ -504,6 +665,9 @@ namespace System.Data {
 			// now we check if any ForeignKeyConstraint is referncing 'table'.
 			if (DataSet != null)
 			{
+				if (DataSet._xmlDataDocument != null) 
+					throw new NotSupportedException ("Clear function on dataset and datatable is not supported on XmlDataDocument.");
+
 				IEnumerator tableEnumerator = DataSet.Tables.GetEnumerator();
 			
 				// loop on all tables in dataset
@@ -525,8 +689,6 @@ namespace System.Data {
 				}
 			}
 			
-			// TODO: throw a NotSupportedException if the DataTable is part
-			//       of a DataSet that is binded to an XmlDataDocument
 			
 			_rows.Clear ();
 		}
@@ -535,7 +697,6 @@ namespace System.Data {
 		/// Clones the structure of the DataTable, including
 		///  all DataTable schemas and constraints.
 		/// </summary>
-		[MonoTODO]
 		public virtual DataTable Clone () 
 		{
 			DataTable Copy = new DataTable ();			
@@ -551,18 +712,14 @@ namespace System.Data {
 		[MonoTODO]
 		public object Compute (string expression, string filter) 
 		{
-			// TODO: implement this function based
-			//       on Select (expression)
-			//
 			// expression is an aggregate function
 			// filter is an expression used to limit rows
 
-			object obj = null;
-
 			DataRow[] rows = Select(filter);
 			
-			ExpressionAggregate Expression = new ExpressionAggregate (expression);
-			obj = Expression.Result(rows);
+			Parser parser = new Parser (rows);
+			IExpression expr = parser.Compile (expression);
+			object obj = expr.Eval (rows[0]);
 			
 			return obj;
 		}
@@ -570,7 +727,6 @@ namespace System.Data {
 		/// <summary>
 		/// Copies both the structure and data for this DataTable.
 		/// </summary>
-		[MonoTODO]	
 		public DataTable Copy () 
 		{
 			DataTable Copy = new DataTable ();
@@ -588,7 +744,6 @@ namespace System.Data {
 			return Copy;
 		}
 
-		[MonoTODO]
 		private void CopyProperties (DataTable Copy) 
 		{
 					
@@ -601,7 +756,13 @@ namespace System.Data {
 			// Copy.DefaultView
 			// Copy.DesignMode
 			Copy.DisplayExpression = DisplayExpression;
-			// Copy.ExtendedProperties
+			if(ExtendedProperties.Count > 0) {
+				//  Cannot copy extended properties directly as the property does not have a set accessor
+				Array tgtArray = Array.CreateInstance( typeof (object), ExtendedProperties.Count);
+				ExtendedProperties.Keys.CopyTo (tgtArray, 0);
+				for (int i=0; i < ExtendedProperties.Count; i++)
+					Copy.ExtendedProperties.Add (tgtArray.GetValue (i), ExtendedProperties[tgtArray.GetValue (i)]);
+			}
 			Copy.Locale = Locale;
 			Copy.MinimumCapacity = MinimumCapacity;
 			Copy.Namespace = Namespace;
@@ -654,8 +815,13 @@ namespace System.Data {
 		/// initialization occurs at runtime.
 		/// </summary>
 		[MonoTODO]
-		public void EndInit () 
+		public virtual void EndInit () 
 		{
+			 _initStatus = initStatus.EndInit;
+                        // Add the constraints
+                        PostEndInit _postEndInit = new PostEndInit (_constraintCollection.PostEndInit);
+                        _postEndInit();
+
 
 		}
 
@@ -663,29 +829,30 @@ namespace System.Data {
 		/// Turns on notifications, index maintenance, and 
 		/// constraints after loading data.
 		/// </summary>
-		[MonoTODO]
 		public void EndLoadData() 
 		{
 			int i = 0;
 			if (this._duringDataLoad) 
 			{
-				//Returning from loading mode, raising exceptions as usual
-				this._duringDataLoad = false;
 				
 				if (this.dataSet !=null)
 				{
 					//Getting back to previous EnforceConstraint state
 					this.dataSet.EnforceConstraints = this.dataSetPrevEnforceConstraints;
 				}
-				for (i=0 ; i<this.Rows.Count ; i++)
-				{
-					if (this.Rows[i]._nullConstraintViolation )
-					{
-						throw new ConstraintException ("Failed to enable constraints. One or more rows contain values violating non-null, unique, or foreign-key constraints.");
-					}
-						
+				else {
+					//Getting back to the table's previous EnforceConstraint state
+					this.EnforceConstraints = this.dataTablePrevEnforceConstraints;
 				}
 
+				if(this._nullConstraintViolationDuringDataLoad) {
+					this._nullConstraintViolationDuringDataLoad = false;
+					throw new ConstraintException ("Failed to enable constraints. One or more rows contain values violating non-null, unique, or foreign-key constraints.");
+				}
+
+				//Returning from loading mode, raising exceptions as usual
+				this._duringDataLoad = false;
+
 			}
 
 		}
@@ -695,7 +862,6 @@ namespace System.Data {
 		///  changes made to it since it was loaded or 
 		///  AcceptChanges was last called.
 		/// </summary>
-		[MonoTODO]
 		public DataTable GetChanges() 
 		{
 			return GetChanges(DataRowState.Added | DataRowState.Deleted | DataRowState.Modified);
@@ -706,7 +872,6 @@ namespace System.Data {
 		/// changes made to it since it was last loaded, or 
 		/// since AcceptChanges was called, filtered by DataRowState.
 		/// </summary>
-		[MonoTODO]	
 		public DataTable GetChanges(DataRowState rowStates) 
 		{
 			DataTable copyTable = null;
@@ -714,19 +879,21 @@ namespace System.Data {
 			IEnumerator rowEnumerator = Rows.GetEnumerator();
 			while (rowEnumerator.MoveNext()) {
 				DataRow row = (DataRow)rowEnumerator.Current;
+				// The spec says relationship constraints may cause Unchanged parent rows to be included but
+				// MS .NET 1.1 does not include Unchanged rows even if their child rows are changed.
 				if (row.IsRowChanged(rowStates)) {
 					if (copyTable == null)
 						copyTable = Clone();
 					DataRow newRow = copyTable.NewRow();
-					copyTable.Rows.Add(newRow);
 					row.CopyValuesToRow(newRow);
+					copyTable.Rows.Add (newRow);
 				}
 			}
 			 
 			return copyTable;
 		}
 
-#if NET_1_2
+#if NET_2_0
 		[MonoTODO]
 		public DataTableReader GetDataReader ()
 		{
@@ -737,7 +904,6 @@ namespace System.Data {
 		/// <summary>
 		/// Gets an array of DataRow objects that contain errors.
 		/// </summary>
-		[MonoTODO]
 		public DataRow[] GetErrors () 
 		{
 			ArrayList errors = new ArrayList();
@@ -784,7 +950,6 @@ namespace System.Data {
 		/// Copies a DataRow into a DataTable, preserving any 
 		/// property settings, as well as original and current values.
 		/// </summary>
-		[MonoTODO]
 		public void ImportRow (DataRow row) 
 		{
 			DataRow newRow = NewRow();
@@ -796,12 +961,33 @@ namespace System.Data {
 		/// <summary>
 		/// This member is only meant to support Mono's infrastructure 		
 		/// </summary>
-		[MonoTODO]
 		void ISerializable.GetObjectData (SerializationInfo info, StreamingContext context) 
 		{
+			DataSet dset;
+			if (dataSet != null)
+				dset = dataSet;
+			else {
+				dset = new DataSet ("tmpDataSet");
+				dset.Tables.Add (this);
+			}
+			
+			StringWriter sw = new StringWriter ();
+			XmlTextWriter tw = new XmlTextWriter (sw);
+			dset.WriteIndividualTableContent (tw, this, XmlWriteMode.DiffGram);
+			tw.Close ();
+			
+			StringWriter sw2 = new StringWriter ();
+			DataTableCollection tables = new DataTableCollection (dset);
+			tables.Add (this);
+			XmlSchema schema = dset.BuildSchema (tables, null);
+			schema.Write (sw2);
+			sw2.Close ();
+			
+			info.AddValue ("XmlSchema", sw2.ToString(), typeof(string));
+			info.AddValue ("XmlDiffGram", sw.ToString(), typeof(string));
 		}
 
-#if NET_1_2
+#if NET_2_0
 		[MonoTODO]
 		public void Load (IDataReader reader)
 		{
@@ -819,7 +1005,6 @@ namespace System.Data {
 		/// Finds and updates a specific row. If no matching row
 		///  is found, a new row is created using the given values.
 		/// </summary>
-		[MonoTODO]
 		public DataRow LoadDataRow (object[] values, bool fAcceptChanges) 
 		{
 			DataRow row = null;
@@ -857,7 +1042,59 @@ namespace System.Data {
 			return row;
 		}
 
-#if NET_1_2
+		internal DataRow LoadDataRow(IDataRecord record, int[] mapping, bool fAcceptChanges)
+		{
+			DataRow row = null;
+			if (PrimaryKey.Length == 0) {
+				row = NewRow();
+				row.SetValuesFromDataRecord(record, mapping);
+				Rows.Add (row);
+
+				if (fAcceptChanges) {
+					row.AcceptChanges();
+				}
+			}
+			else {
+				bool hasPrimaryValues = true;
+				int tmpRecord = this.RecordCache.NewRecord();
+				try {
+					for (int i = 0; i < PrimaryKey.Length && hasPrimaryValues; i++) {
+						DataColumn primaryKeyColumn = PrimaryKey[i];
+						int ordinal = primaryKeyColumn.Ordinal;
+						if(ordinal < mapping.Length) {
+							primaryKeyColumn.DataContainer.SetItemFromDataRecord(tmpRecord,record,mapping[ordinal]);
+						}
+						else {
+							hasPrimaryValues = false;
+						}
+					}
+					
+					if (hasPrimaryValues) {
+						// find the row in the table.
+						row = Rows.Find(tmpRecord,PrimaryKey.Length);
+					}
+				}
+				finally {
+					this.RecordCache.DisposeRecord(tmpRecord);
+				}
+
+				if (row == null) {
+					row = NewRow();
+					row.SetValuesFromDataRecord(record, mapping);
+					Rows.Add (row);
+				}
+				else {
+					row.SetValuesFromDataRecord(record, mapping);
+				}
+				
+				if (fAcceptChanges) {
+					row.AcceptChanges();
+				}
+			}				
+			return row;
+		}
+
+#if NET_2_0
 		[MonoTODO]
 		public DataRow LoadDataRow (object[] values, LoadOption loadOption)
 		{
@@ -888,7 +1125,13 @@ namespace System.Data {
 		/// </summary>
 		public DataRow NewRow () 
 		{
-			return this.NewRowFromBuilder (new DataRowBuilder (this, 0, 0));
+			// initiate only one row builder.
+			if (_rowBuilder == null)
+				_rowBuilder = new DataRowBuilder (this, 0, 0);
+			
+			// new row get id -1.
+			_rowBuilder._rowId = -1;
+			return this.NewRowFromBuilder (_rowBuilder);
 		}
 
 		/// <summary>
@@ -907,36 +1150,39 @@ namespace System.Data {
 		{
 			return new DataRow (builder);
 		}
+		
+		
 
-#if NET_1_2
+#if NET_2_0
 		[MonoTODO]
 		XmlReadMode ReadXml (Stream stream)
 		{
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
 		public void ReadXmlSchema (Stream stream)
 		{
-			throw new NotImplementedException ();
+			XmlSchemaMapper mapper = new XmlSchemaMapper (this);
+			mapper.Read (new XmlTextReader(stream));
 		}
 
-		[MonoTODO]
 		public void ReadXmlSchema (TextReader reader)
 		{
-			throw new NotImplementedException ();
+			XmlSchemaMapper mapper = new XmlSchemaMapper (this);
+			mapper.Read (new XmlTextReader(reader));
 		}
 
-		[MonoTODO]
 		public void ReadXmlSchema (string fileName)
 		{
-			throw new NotImplementedException ();
+			StreamReader reader = new StreamReader (fileName);
+			ReadXmlSchema (reader);
+			reader.Close ();
 		}
 
-		[MonoTODO]
 		public void ReadXmlSchema (XmlReader reader)
 		{
-			throw new NotImplementedException ();
+			XmlSchemaMapper mapper = new XmlSchemaMapper (this);
+			mapper.Read (reader);
 		}
 #endif
 
@@ -945,7 +1191,6 @@ namespace System.Data {
 		/// table since it was loaded, or the last time AcceptChanges
 		///  was called.
 		/// </summary>
-		[MonoTODO]
 		public void RejectChanges () 
 		{	
 			for (int i = _rows.Count - 1; i >= 0; i--) {
@@ -958,7 +1203,6 @@ namespace System.Data {
 		/// <summary>
 		/// Resets the DataTable to its original state.
 		/// </summary>		
-		[MonoTODO]
 		public virtual void Reset () 
 		{
 			Clear();
@@ -1013,27 +1257,26 @@ namespace System.Data {
 		[MonoTODO]
 		public DataRow[] Select(string filterExpression, string sort, DataViewRowState recordStates) 
 		{
-			DataRow[] dataRows = null;
 			if (filterExpression == null)
 				filterExpression = String.Empty;
 
-			ExpressionElement Expression = null;
-			if (filterExpression != null && filterExpression.Length != 0)
-				Expression = new ExpressionMainElement(filterExpression);
+			IExpression filter = null;
+			if (filterExpression != String.Empty) {
+				Parser parser = new Parser ();
+				filter = parser.Compile (filterExpression);
+			}
 
-			ArrayList List = new ArrayList();
+			ArrayList rowList = new ArrayList();
 			int recordStateFilter = GetRowStateFilter(recordStates);
-			foreach (DataRow Row in Rows) 
-			{
-				if (((int)Row.RowState & recordStateFilter) != 0)
-				{
-					if (Expression == null || Expression.Test(Row))
-						List.Add(Row);
+			foreach (DataRow row in Rows) {
+				if (((int)row.RowState & recordStateFilter) != 0) {
+					if (filter != null && !(bool)filter.Eval (row))
+						continue;
+					rowList.Add (row);
 				}
 			}
 
-			dataRows = (DataRow[])List.ToArray(typeof(DataRow));
-			
+			DataRow[] dataRows = (DataRow[])rowList.ToArray(typeof(DataRow));
 
 			if (sort != null && !sort.Equals(String.Empty)) 
 			{
@@ -1045,7 +1288,7 @@ namespace System.Data {
 				if (sortableColumns.Length == 0)
 					throw new Exception("sort expression result is 0");
 
-				RowSorter rowSorter = new RowSorter (dataRows, sortableColumns);
+				RowSorter rowSorter = new RowSorter (this, dataRows, sortableColumns);
 				dataRows = rowSorter.SortRows ();
 
 				sortableColumns = null;
@@ -1056,6 +1299,82 @@ namespace System.Data {
 			return dataRows;
 		}
 
+		internal void AddIndex (Index index)
+		{
+			if (_indexes == null)
+				_indexes = new ArrayList();
+
+			_indexes.Add (index);
+		}
+
+		internal void RemoveIndex (Index indx)
+		{
+			_indexes.Remove (indx);
+		}
+
+		internal Index GetIndexByColumns (DataColumn[] columns)
+		{
+			return GetIndexByColumns(columns,false,false);
+		}
+
+//		internal Index GetIndexByColumnsExtended(DataColumn[] columns)
+//		{
+//			DataColumn[] pkColumns = this.PrimaryKey;
+//			if((pkColumns != null) && (pkColumns.Length > 0)) {
+//				DataColumn[] cols = new DataColumn[columns.Length + pkColumns.Length];					
+//				Array.Copy(columns,0,cols,0,columns.Length);
+//				Array.Copy(pkColumns,0,cols,columns.Length,pkColumns.Length);
+//
+//				return _getIndexByColumns(cols,false,false);
+//			} else {
+//				return null;
+//			}
+//		}
+
+		internal Index GetIndexByColumns (DataColumn[] columns, bool unique)
+		{
+			return GetIndexByColumns(columns,unique,true);
+		}
+
+//		internal Index GetIndexByColumnsExtended(DataColumn[] columns, bool unique)
+//		{
+//			DataColumn[] pkColumns = this.PrimaryKey;
+//			if((pkColumns != null) && (pkColumns.Length > 0)) {
+//				DataColumn[] cols = new DataColumn[columns.Length + pkColumns.Length];					
+//				Array.Copy(columns,0,cols,0,columns.Length);
+//				Array.Copy(pkColumns,0,cols,columns.Length,pkColumns.Length);
+//
+//				return _getIndexByColumns(cols,unique,true);
+//			} else  {
+//				return null;
+//			}
+//		}
+
+		internal Index GetIndexByColumns(DataColumn[] columns, bool unique, bool useUnique)
+		{
+			if (_indexes != null) {
+				foreach (Index indx in _indexes) {
+					bool found = false;
+					if ((!useUnique) || ((useUnique)&& (indx.IsUnique))) {
+						found = DataColumn.AreColumnSetsTheSame (indx.Columns, columns);
+					}
+					if (found)
+						return indx;
+				}
+			}
+
+			return null;
+		}
+
+		internal void DeleteRowFromIndexes (DataRow row)
+		{
+			if (_indexes != null) {
+				foreach (Index indx in _indexes) {
+					indx.Delete (row);
+				}
+			}
+		}
+
 		private static int GetRowStateFilter(DataViewRowState recordStates)
 		{
 			int flag = 0;
@@ -1088,7 +1407,7 @@ namespace System.Data {
 			return retVal;
 		}
 
-#if NET_1_2
+#if NET_2_0
 		[MonoTODO]
 		public void WriteXml (Stream stream)
 		{
@@ -1162,7 +1481,7 @@ namespace System.Data {
 		}
 #endif
 		
-		#region Events /////////////////
+		#region Events 
 		
 		/// <summary>
 		/// Raises the ColumnChanged event.
@@ -1173,6 +1492,10 @@ namespace System.Data {
 			}
 		}
 
+		internal void RaiseOnColumnChanged (DataColumnChangeEventArgs e) {
+			OnColumnChanged(e);
+		}
+
 		/// <summary>
 		/// Raises the ColumnChanging event.
 		/// </summary>
@@ -1202,6 +1525,7 @@ namespace System.Data {
 			//	{
 			//		RemoveColumn(this, e);
 			//	}
+			this.Rows.onColumnRemoved(column.Ordinal);
 		}
 
 		/// <summary>
@@ -1349,7 +1673,7 @@ namespace System.Data {
 					string columnName = columnSortInfo[0].Trim ();
 					string sortOrder = "ASC";
 					if (columnSortInfo.Length > 1) 
-						sortOrder = columnSortInfo[1].Trim ().ToUpper ();
+						sortOrder = columnSortInfo[1].Trim ().ToUpper (Locale);
 					
 					ListSortDirection sortDirection = ListSortDirection.Ascending;
 					switch (sortOrder) {
@@ -1409,12 +1733,15 @@ namespace System.Data {
 
 		private class RowSorter : IComparer 
 		{
+			private DataTable table;
 			private SortableColumn[] sortColumns;
 			private DataRow[] rowsToSort;
 			
-			internal RowSorter(DataRow[] unsortedRows, 
+			internal RowSorter(DataTable table,
+					DataRow[] unsortedRows, 
 					SortableColumn[] sortColumns) 
 			{
+				this.table = table;
 				this.sortColumns = sortColumns;
 				this.rowsToSort = unsortedRows;
 			}
@@ -1449,10 +1776,10 @@ namespace System.Data {
 					SortableColumn sortColumn = sortColumns[i];
 					DataColumn dc = sortColumn.Column;
 
-					IComparable objx = (IComparable) rowx[dc];
-					object objy = rowy[dc];
+					int result = dc.CompareValues(
+						rowx.IndexFromVersion(DataRowVersion.Default),
+						rowy.IndexFromVersion(DataRowVersion.Default));
 
-					int result = CompareObjects (objx, objy);
 					if (result != 0) {
 						if (sortColumn.SortDirection == ListSortDirection.Ascending) {
 							return result;
@@ -1464,31 +1791,106 @@ namespace System.Data {
 				}
 				return 0;
 			}
+		}
 
-			private int CompareObjects (object a, object b) 
-			{
-				if (a == b)
-					return 0;
-				else if (a == null)
-					return -1;
-				else if (a == DBNull.Value)
-					return -1;
-				else if (b == null)
-					return 1;
-				else if (b == DBNull.Value)
-					return 1;
+		/// <summary>
+		/// Creates new index for a table
+		/// </summary>
+		internal Index CreateIndex(string name, DataColumn[] columns, bool unique)
+		{
+			// first check whenever index exists on the columns
+			Index idx = this.GetIndexByColumns(columns);
+			if(idx != null) {
+			// if index on this columns already exists - return it
+				return idx;
+			}
 
-				if((a is string) && (b is string)) {
-					a = ((string) a).ToUpper ();
-					b = ((string) b).ToUpper ();			
+			// create new index
+			Index newIndex = new Index(name,this,columns,unique);
+
+			//InitializeIndex (newIndex);			
+
+			// add new index to table indexes
+			this.AddIndex(newIndex);
+			return newIndex;
+		}
+
+//		/// <summary>
+//		/// Creates new extended index for a table
+//		/// </summary>
+//		internal Index CreateIndexExtended(string name, DataColumn[] columns, bool unique)
+//		{
+//			// first check whenever extended index exists on the columns
+//			Index idx = this.GetIndexByColumnsExtended(columns);
+//			if(idx != null) {
+//				// if extended index on this columns already exists - return it
+//				return idx;
+//			}
+//
+//			DataColumn[] pkColumns = this.PrimaryKey;
+//			if((pkColumns != null) && (pkColumns.Length > 0)) {
+//				DataColumn[] cols = new DataColumn[columns.Length + pkColumns.Length];					
+//				Array.Copy(columns,0,cols,0,columns.Length);
+//				Array.Copy(pkColumns,0,cols,columns.Length,pkColumns.Length);
+//				return this.CreateIndex(name, cols, unique);
+//			} 
+//			else {
+//				throw new InvalidOperationException("Can not create extended index if the primary key is null or primary key does not contains any row");
+//			}
+//		}
+
+//		/// <summary>
+//		/// Drops extended index if it is not referenced anymore
+//		/// by any of table constraints
+//		/// </summary>
+//		internal void DropIndexExtended(DataColumn[] columns)
+//		{
+//			// first check whenever extended index exists on the columns
+//			Index index = this.GetIndexByColumnsExtended(columns);
+//			if(index == null) {
+//				// if no extended index on this columns exists - do nothing
+//				return;
+//			}
+//			this.DropIndex(index);
+//		}
+
+		/// <summary>
+		/// Drops index specified by columns if it is not referenced anymore
+		/// by any of table constraints
+		/// </summary>
+		internal void DropIndex(DataColumn[] columns)
+		{
+			// first check whenever index exists for the columns
+			Index index = this.GetIndexByColumns(columns);
+			if(index == null) {
+			// if no index on this columns already exists - do nothing
+				return;
+			}
+			this.DropIndex(index);
+		}
+
+		internal void DropIndex(Index index)
+		{
+			// loop through table constraints and checks 
+			foreach(Constraint constraint in Constraints) {
+				// if we found another reference to the index we do not remove the index.
+				if (index == constraint.Index)
+					return; 
+			}
+			
+			this.RemoveIndex(index);
+		}
+
+		internal void InitializeIndex (Index indx)
+		{
+			DataRow[] rows = new DataRow[this.Rows.Count];
+			this.Rows.CopyTo (rows, 0);
+			indx.Root = null;
+			// fill index with table rows
+			foreach(DataRow row in this.Rows) {
+				if(row.RowState != DataRowState.Deleted) {
+					indx.Insert(new Node(row), DataRowVersion.Default);
 				}
-
-				if (a is IComparable)
-					return ((a as IComparable).CompareTo (b));
-				else if (b is IComparable)
-					return -((b as IComparable).CompareTo (a));
-
-				throw new ArgumentException ("Neither a nor b IComparable");
 			}
 		}
 	}
