@@ -39,17 +39,17 @@ namespace System.Data {
 		#region Properties
 
 		public DataTable this[int index] {
-			get { return (DataTable)(list[index]); }
+			get { return (DataTable)(List[index]); }
 		}
 
 		public DataTable this[string name] {
 			get { 
 				int index = IndexOf (name, true);
-				return index < 0 ? null : (DataTable) list[index];
+				return index < 0 ? null : (DataTable) List[index];
 			}
 		}
 
-#if NET_1_2
+#if NET_2_0
 		[MonoTODO]
 		public DataTable this [string name, string tbNamespace] {
 			get { throw new NotImplementedException (); }
@@ -57,7 +57,7 @@ namespace System.Data {
 #endif
 
 		protected override ArrayList List {
-			get { return list; }
+			get { return base.List; }
 		}
 
 		#endregion
@@ -79,7 +79,7 @@ namespace System.Data {
 				throw new ArgumentNullException("table");
             
 			// check if the list already contains this tabe.
-			if(list.Contains(table))
+			if(List.Contains(table))
 				throw new ArgumentException("DataTable already belongs to this DataSet.");
             
 			// if the table name is null or empty string.
@@ -98,7 +98,7 @@ namespace System.Data {
 					throw new DuplicateNameException("A DataTable named '" + table.TableName + "' already belongs to this DataSet.");
 			}
 	
-			list.Add (table);
+			List.Add (table);
 			table.dataSet = dataSet;
 			OnCollectionChanged (new CollectionChangeEventArgs (CollectionChangeAction.Add, table));
 		}
@@ -110,7 +110,7 @@ namespace System.Data {
 			return table;
 		}
 
-#if NET_1_2
+#if NET_2_0
 		public virtual DataTable Add (string name, string tbNamespace)
 		{
 			DataTable table = new DataTable (name, tbNamespace);
@@ -125,7 +125,6 @@ namespace System.Data {
 				this.Add (table);
 		}
 
-		[MonoTODO]
 		public bool CanRemove (DataTable table) 
 		{
 			return CanRemove(table, false);
@@ -133,7 +132,7 @@ namespace System.Data {
 
 		public void Clear () 
 		{
-			list.Clear ();
+			List.Clear ();
 		}
 
 		public bool Contains (string name) 
@@ -143,7 +142,7 @@ namespace System.Data {
 
 		public virtual int IndexOf (DataTable table) 
 		{
-			return list.IndexOf (table);
+			return List.IndexOf (table);
 		}
 
 		public virtual int IndexOf (string name) 
@@ -154,7 +153,7 @@ namespace System.Data {
 		public void Remove (DataTable table) 
 		{
 			CanRemove(table, true);
-			list.Remove(table);
+			List.Remove(table);
 			OnCollectionChanged (new CollectionChangeEventArgs (CollectionChangeAction.Remove, table));
 		}
 
@@ -165,9 +164,11 @@ namespace System.Data {
 
 		public void RemoveAt (int index) 
 		{
-			DataTable t = this[index];
+			if (( index < 0 ) || (index >=List.Count))
+				throw new ArgumentException("There is no row at position "+index+".");
+			DataTable t = this[index];	
 			CanRemove(t, true);
-			list.RemoveAt (index);
+			List.RemoveAt (index);
 			OnCollectionChanged (new CollectionChangeEventArgs (CollectionChangeAction.Remove, t));
 		}
 
@@ -194,9 +195,9 @@ namespace System.Data {
 		private int IndexOf (string name, bool error)
 		{
 			int count = 0, match = -1;
-			for (int i = 0; i < list.Count; i++)
+			for (int i = 0; i < List.Count; i++)
 			{
-				String name2 = ((DataTable) list[i]).TableName;
+				String name2 = ((DataTable) List[i]).TableName;
 				if (String.Compare (name, name2, true) == 0)
 				{
 					if (String.Compare (name, name2, false) == 0)
@@ -230,6 +231,7 @@ namespace System.Data {
 		// if it is true throws an Exception, else return false.
 		private bool CanRemove(DataTable table, bool throwException)
 		{
+
 			// check if table is null reference
 			if (table == null)
 			{
@@ -237,7 +239,30 @@ namespace System.Data {
 					throw new ArgumentNullException("table");
 				return false;
 			}
-			
+
+			// check if the table doesnot belong to this collection
+			bool tableexists = true;
+			int tmp = IndexOf(table.TableName);
+                        // if we found a table with same name we have to check
+                        // that it is the same case.
+                        // indexof can return a table with different case letters.
+                        if (tmp != -1)
+			{
+				 if(table.TableName != this[tmp].TableName)
+					tableexists = false;
+			}
+			else
+				tableexists = false;
+
+			if (!tableexists)
+			{
+				if(throwException)
+                                        throw new ArgumentException("Table does not exist in collection");
+                                return false;
+
+			}		
+				
+						
 			// check if the table has the same DataSet as this collection.
 			if(table.DataSet != this.dataSet)
 			{
