@@ -27,6 +27,7 @@ namespace System.Data
 	[Editor]
 	[DefaultEvent ("PositionChanged")]
 	[DefaultProperty ("Table")]
+	[DesignerAttribute ("Microsoft.VSDesigner.Data.VS.DataViewDesigner, "+ Consts.AssemblyMicrosoft_VSDesigner, "System.ComponentModel.Design.IDesigner")]
 	public class DataView : MarshalByValueComponent, IBindingList, IList, ICollection, IEnumerable, ITypedList, ISupportInitialize
 	{
 		DataTable dataTable = null;
@@ -77,12 +78,9 @@ namespace System.Data
 		[DataSysDescription ("Indicates whether this DataView and the user interface associated with it allows deletes.")]
 		[DefaultValue (true)]
 		public bool AllowDelete {
-			[MonoTODO]
 			get {
 				return allowDelete;
 			}
-			
-			[MonoTODO]
 			set {
 				allowDelete = value;
 			}
@@ -92,12 +90,9 @@ namespace System.Data
 		[DataSysDescription ("Indicates whether this DataView and the user interface associated with it allows edits.")]
 		[DefaultValue (true)]
 		public bool AllowEdit {
-			[MonoTODO]
 			get {
 				return allowEdit;
 			}
-			
-			[MonoTODO]
 			set {
 				allowEdit = value;
 			}
@@ -107,12 +102,10 @@ namespace System.Data
 		[DataSysDescription ("Indicates whether this DataView and the user interface associated with it allows new rows to be added.")]
 		[DefaultValue (true)]
 		public bool AllowNew {
-			[MonoTODO]
 			get {
 				return allowNew;
 			}
 			
-			[MonoTODO]
 			set {
 				allowNew = value;
 			}
@@ -238,6 +231,7 @@ namespace System.Data
 				//        has changes via column, row, or constraint
 				//        changed events
 				UpdateIndex ();
+				OnListChanged (new ListChangedEventArgs (ListChangedType.Reset, - 1, -1));
 			}
 		}
 
@@ -265,7 +259,19 @@ namespace System.Data
 		[MonoTODO]
 		public virtual DataRowView AddNew() 
 		{
-			throw new NotImplementedException ();
+			if (!IsOpen)
+				throw new DataException ("DataView is not open.");
+			if (!AllowNew)
+				throw new DataException ("Cannot call AddNew on a DataView where AllowNew is false.");
+			
+			DataRow row = dataTable.NewRow ();
+			DataRowView rowView = new DataRowView (this, row, true);
+			
+			dataTable.Rows.Add (row);
+
+			UpdateIndex ();
+			OnListChanged (new ListChangedEventArgs (ListChangedType.ItemAdded, rowCache.Length - 1,-1));
+			return rowView;
 		}
 
 		[MonoTODO]
@@ -295,13 +301,26 @@ namespace System.Data
 			}
 		}
 
-		[MonoTODO]
 		public void Delete(int index) 
 		{
-			throw new NotImplementedException ();
+			if (!IsOpen)
+				throw new DataException ("DataView is not open.");
+			if (!AllowDelete)
+				throw new DataException ("Cannot delete on a DataSource where AllowDelete is false.");
+			
+			UpdateIndex ();
+
+			if (index > rowCache.Length)
+				throw new IndexOutOfRangeException ("There is no row at " +
+						"position: " + index + ".");
+			
+			DataRowView row = rowCache [index];
+			row.Row.Delete ();
+
+			OnListChanged (new ListChangedEventArgs (ListChangedType.ItemDeleted, index));
 		}
 
-#if NET_1_2
+#if NET_2_0
 		[MonoTODO]
 		public virtual bool Equals (DataView dv)
 		{
@@ -415,7 +434,17 @@ namespace System.Data
 		[MonoTODO]
 		protected virtual void OnListChanged(ListChangedEventArgs e) 
 		{
-			throw new NotImplementedException ();
+			try {
+				if (ListChanged != null)
+					ListChanged (this, e);
+			} catch {
+			}
+		}
+
+		internal void ChangedList(ListChangedType listChangedType, int newIndex,int oldIndex)
+		{
+			ListChangedEventArgs e = new ListChangedEventArgs(listChangedType,newIndex,oldIndex);
+			OnListChanged(e);
 		}
 
 		[MonoTODO]
@@ -450,9 +479,10 @@ namespace System.Data
 			Close ();
 			rowCache = null;
 			Open ();
+			OnListChanged (new ListChangedEventArgs (ListChangedType.Reset, -1));
 		}
 
-#if NET_1_2
+#if NET_2_0
 		[MonoTODO]
 		public DataTable ToTable ()
 		{
@@ -512,7 +542,8 @@ namespace System.Data
 					dataColumn.ColumnName, col, null);
 				descriptor.SetComponentType (typeof (System.Data.DataRowView));
 				descriptor.SetPropertyType (dataColumn.DataType);
-				
+				descriptor.SetReadOnly (dataColumn.ReadOnly);
+
 				descriptors[col] = descriptor;
 			}
 
