@@ -9,10 +9,13 @@
 using System;
 using System.IO;
 using System.Reflection;
+using System.Runtime.InteropServices;
 using System.Runtime.Remoting;
 using System.Runtime.Serialization;
 using System.Runtime.Remoting.Messaging;
 using System.Xml.Serialization;
+using System.Threading;
+using System.Globalization;
 
 
 namespace System.Runtime.Serialization.Formatters.Soap {
@@ -20,24 +23,25 @@ namespace System.Runtime.Serialization.Formatters.Soap {
 		MethodCall, MethodResponse, ServerFault, NotRecognize
 	}
 	
-	public class SoapFormatter: IRemotingFormatter, IFormatter {
-		private ObjectWriter _objWriter;
+	public sealed class SoapFormatter: IRemotingFormatter, IFormatter {
 		private SoapWriter _soapWriter;
 		private SerializationBinder _binder;
 		private StreamingContext _context;
 		private ISurrogateSelector _selector;
 		private FormatterAssemblyStyle _assemblyFormat = FormatterAssemblyStyle.Full;
-		private ISoapMessage _topObject;
+		private FormatterTypeStyle _typeFormat = FormatterTypeStyle.TypesWhenNeeded;
+		private ISoapMessage _topObject = null;
 		
 #if NET_1_1
 		TypeFilterLevel _filterLevel = TypeFilterLevel.Low;
 #endif
 
 		public SoapFormatter() {
-			
+			_selector = null;
+			_context = new StreamingContext(StreamingContextStates.All);
 		}
 		
-		public SoapFormatter(ISurrogateSelector selector, StreamingContext context):this() {
+		public SoapFormatter(ISurrogateSelector selector, StreamingContext context) {
 			_selector = selector;
 			_context = context;
 		}
@@ -51,15 +55,16 @@ namespace System.Runtime.Serialization.Formatters.Soap {
 		}
 		
 		public object Deserialize(Stream serializationStream, HeaderHandler handler) {
-			SoapParser parser = new SoapParser(serializationStream);
-			SoapReader soapReader = new SoapReader(parser);
-			soapReader.Binder = _binder;
-			
-			
-			if(_topObject != null) soapReader.TopObject = _topObject;
-			ObjectReader reader = new ObjectReader(_selector, _context, soapReader);
-			parser.Run();
-			object objReturn = reader.TopObject;
+			object objReturn = null;
+			SoapReader soapReader = new SoapReader(_binder, _selector, _context);
+			CultureInfo savedCi = CultureInfo.CurrentCulture;
+			try {
+				Thread.CurrentThread.CurrentCulture = new CultureInfo("en-US");
+				objReturn = soapReader.Deserialize(serializationStream, _topObject);
+			}
+			finally {
+				Thread.CurrentThread.CurrentCulture = savedCi;
+			}
 			return objReturn;
 		}
 		
@@ -74,10 +79,17 @@ namespace System.Runtime.Serialization.Formatters.Soap {
 				throw new ArgumentNullException("serializationStream");
 			if(!serializationStream.CanWrite)
 				throw new SerializationException("Can't write in the serialization stream");
-			_soapWriter = new SoapWriter(serializationStream);
-			_objWriter = new ObjectWriter((ISoapWriter) _soapWriter, _selector,  new StreamingContext(StreamingContextStates.File));
-			_soapWriter.Writer = _objWriter;
-			_objWriter.Serialize(graph);
+			if(graph == null)
+				throw new ArgumentNullException("graph");
+			_soapWriter = new SoapWriter(serializationStream, _selector, _context, _topObject);
+			CultureInfo savedCi = CultureInfo.CurrentCulture;
+			try {
+				Thread.CurrentThread.CurrentCulture = new CultureInfo("en-US");
+				_soapWriter.Serialize (graph, headers, _typeFormat, _assemblyFormat);
+			}
+			finally {
+				Thread.CurrentThread.CurrentCulture = savedCi;
+			}
 			
 		}
 		
@@ -120,6 +132,7 @@ namespace System.Runtime.Serialization.Formatters.Soap {
 		
 #if NET_1_1
 		[MonoTODO ("Interpret this")]
+		[ComVisible(false)]
 		public TypeFilterLevel FilterLevel {
 			get {
 				return _filterLevel;
@@ -130,7 +143,6 @@ namespace System.Runtime.Serialization.Formatters.Soap {
 		}
 #endif
 		
-		[MonoTODO ("Interpret this")]
 		public FormatterAssemblyStyle AssemblyFormat
 		{
 			get {
@@ -141,5 +153,17 @@ namespace System.Runtime.Serialization.Formatters.Soap {
 			}
 		}
 
+		public FormatterTypeStyle TypeFormat
+		{
+			get 
+			{
+				return _typeFormat;
+			}
+			set 
+			{
+				_typeFormat = value;
+			}
+		}
+
 	}
 }
