@@ -2,12 +2,15 @@
 // Transform.cs - Transform implementation for XML Signature
 //
 // Author:
-//	Sebastien Pouliot (spouliot@motus.com)
+//	Sebastien Pouliot <sebastien@ximian.com>
+//	Atsushi Enomoto <atsushi@ximian.com>
 //
 // (C) 2002, 2003 Motus Technologies Inc. (http://www.motus.com)
+// (C) 2004 Novell (http://www.novell.com)
 //
 
 using System.Runtime.InteropServices;
+using System.Security.Policy;
 using System.Xml;
 
 namespace System.Security.Cryptography.Xml { 
@@ -15,8 +18,17 @@ namespace System.Security.Cryptography.Xml {
 	public abstract class Transform {
 
 		private string algo;
+		private XmlResolver xmlResolver;
 
-		public Transform () {}
+		public Transform ()
+		{
+		// FIXME: enable it after CAS implementation
+#if false // NET_1_1
+			xmlResolver = new XmlSecureResolver (new XmlUrlResolver (), (Evidence) new Evidence ());
+#else
+			xmlResolver = new XmlUrlResolver ();
+#endif
+		}
 
 		public string Algorithm {
 			get { return algo; }
@@ -40,8 +52,16 @@ namespace System.Security.Cryptography.Xml {
 		public XmlElement GetXml () 
 		{
 			XmlDocument document = new XmlDocument ();
+			document.XmlResolver = GetResolver ();
 			XmlElement xel = document.CreateElement (XmlSignature.ElementNames.Transform, XmlSignature.NamespaceURI);
 			xel.SetAttribute (XmlSignature.AttributeNames.Algorithm, algo);
+			XmlNodeList xnl = this.GetInnerXml ();
+			if (xnl != null) {
+				foreach (XmlNode xn in xnl) {
+					XmlNode importedNode = document.ImportNode (xn, true);
+					xel.AppendChild (importedNode);
+				}
+			}
 			return xel;
 		}
 
@@ -49,12 +69,14 @@ namespace System.Security.Cryptography.Xml {
 
 		public abstract void LoadInput (object obj);
 
-#if ! NET_1_0
-		private XmlResolver xmlResolver;
+		internal XmlResolver GetResolver ()
+		{
+			return xmlResolver;
+		}
 
-		[MonoTODO("property not (yet) used in derived classes")]
+#if NET_1_1
 		[ComVisible(false)]
-		XmlResolver Resolver {
+		public XmlResolver Resolver {
 			set { xmlResolver = value; }
 		}
 #endif
